import unittest
import textwrap
import httpretty
import json
import requests
from sure import expect

from six import u, PY2, PY3
from expects import *
from opentok import Client, Broadcast, __version__, BroadcastError
from opentok.broadcast import BroadcastStreamModes
from opentok.exceptions import BroadcastHLSOptionsError
from .validate_jwt import validate_jwt_header


class OpenTokBroadcastTest(unittest.TestCase):
    def setUp(self):
        self.api_key = u("123456")
        self.api_secret = u("1234567890abcdef1234567890abcdef1234567890")
        self.opentok = Client(self.api_key, self.api_secret)
        self.session_id = u("2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4")
        self.stream1 = "kjhk-09898-kjn"
        self.stream2 = "jnn-99kjk-88734r"

    @httpretty.activate
    def test_start_broadcast(self):
        """
        Test start_broadcast() method
        """
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/broadcast").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "id": "1748b7070a81464c9759c46ad10d3734",
                            "sessionId": "2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4",
                            "projectId": 100,
                            "createdAt": 1437676551000,
                            "updatedAt": 1437676551000,
                            "resolution": "640x480",
                            "status": "started",
                            "hasAudio": true,
                            "hasVideo": true,
                            "maxBitrate": 1000000,
                            "maxDuration": 5400,
                            "broadcastUrls": {
                                "hls" : "http://server/fakepath/playlist.m3u8",
                                "hlsStatus": "ready",
                                "rtmp": {
                                    "foo": {
                                        "serverUrl": "rtmp://myfooserver/myfooapp",
                                        "streamName": "myfoostream"
                                    },
                                    "bar": {
                                        "serverUrl": "rtmp://mybarserver/mybarapp",
                                        "streamName": "mybarstream"
                                    }
                                }
                            }
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        options = {
            "layout": {
                "type": "custom",
                "stylesheet": "the layout stylesheet (only used with type == custom)",
            },
            "maxDuration": 5400,
            "maxBitrate": 1000000,
            "outputs": {
                "hls": {},
                "rtmp": [
                    {
                        "id": "foo",
                        "serverUrl": "rtmp://myfooserver/myfooapp",
                        "streamName": "myfoostream",
                    },
                    {
                        "id": "bar",
                        "serverUrl": "rtmp://mybarserver/mybarapp",
                        "streamName": "mybarstream",
                    },
                ],
            },
            "resolution": "640x480",
        }

        broadcast = self.opentok.start_broadcast(self.session_id, options)
        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        # non-deterministic json encoding. have to decode to test it properly
        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))

        expect(body).to(have_key(u("layout")))
        expect(broadcast).to(be_an(Broadcast))
        expect(broadcast).to(
            have_property(u("id"), u("1748b7070a81464c9759c46ad10d3734"))
        )
        expect(broadcast).to(
            have_property(u("sessionId"), u("2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4"))
        )
        expect(broadcast).to(have_property(u("projectId"), 100))
        expect(broadcast).to(have_property(u("createdAt"), 1437676551000))
        expect(broadcast).to(have_property(u("updatedAt"), 1437676551000))
        expect(broadcast).to(have_property(u("resolution"), u("640x480")))
        expect(broadcast).to(have_property(u("status"), u("started")))
        expect(broadcast).to(have_property("maxDuration", 5400))
        expect(broadcast).to(have_property("hasAudio", True))
        expect(broadcast).to(have_property("hasVideo", True))
        expect(broadcast).to(have_property("maxBitrate", 1000000))
        expect(broadcast.broadcastUrls["hlsStatus"]).to(equal("ready"))
        expect(list(broadcast.broadcastUrls)).to(have_length(3))
        expect(list(broadcast.broadcastUrls["rtmp"])).to(have_length(2))

    @httpretty.activate
    def test_start_broadcast_only_one_rtmp(self):
        """
        Test start_broadcast() method with only one rtmp
        """
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/broadcast").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "id": "1748b7070a81464c9759c46ad10d3734",
                            "sessionId": "2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4",
                            "projectId": 100,
                            "createdAt": 1437676551000,
                            "updatedAt": 1437676551000,
                            "resolution": "640x480",
                            "status": "started",
                            "hasAudio": true,
                            "hasVideo": true,
                            "maxBitrate": 1000000,
                            "maxDuration": 5400,
                            "broadcastUrls": {
                                "hls": "http://server/fakepath/playlist.m3u8",
                                "hlsStatus": "connecting",
                                "rtmp": {
                                    "foo": {
                                        "serverUrl": "rtmp://myfooserver/myfooapp",
                                        "streamName": "myfoostream"
                                    },
                                    "bar": {
                                        "serverUrl": "rtmp://mybarserver/mybarapp",
                                        "streamName": "mybarstream"
                                    }
                                }
                            }
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        options = {
            "sessionId": "2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ",
            "layout": {
                "type": "custom",
                "stylesheet": "the layout stylesheet (only used with type == custom)",
            },
            "maxDuration": 5400,
            "outputs": {
                "rtmp": {
                    "id": "my-id",
                    "serverUrl": "rtmp://myserver/myapp",
                    "streamName": "my-stream-name",
                }
            },
            "resolution": "640x480",
        }

        broadcast = self.opentok.start_broadcast(self.session_id, options)
        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        expect(broadcast).to(be_an(Broadcast))
        expect(broadcast).to(
            have_property(u("id"), u("1748b7070a81464c9759c46ad10d3734"))
        )
        expect(broadcast).to(
            have_property(u("sessionId"), u("2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4"))
        )
        expect(broadcast).to(have_property(u("projectId"), 100))
        expect(broadcast).to(have_property(u("createdAt"), 1437676551000))
        expect(broadcast).to(have_property(u("updatedAt"), 1437676551000))
        expect(broadcast).to(have_property(u("resolution"), u("640x480")))
        expect(broadcast).to(have_property(u("status"), u("started")))
        expect(broadcast.broadcastUrls["hlsStatus"]).to(equal("connecting"))
        expect(list(broadcast.broadcastUrls)).to(have_length(3))
        expect(list(broadcast.broadcastUrls["rtmp"])).to(have_length(2))

    @httpretty.activate
    def test_start_broadcast_with_screenshare_type(self):
        """
        Test start_broadcast() method
        """
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/broadcast").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "id": "1748b7070a81464c9759c46ad10d3734",
                            "sessionId": "2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4",
                            "projectId": 100,
                            "createdAt": 1437676551000,
                            "updatedAt": 1437676551000,
                            "resolution": "640x480",
                            "status": "started",
                            "hasAudio": true,
                            "hasVideo": true,
                            "maxBitrate": 1000000,
                            "maxDuration": 5400,
                            "broadcastUrls": {
                                "hls" : "http://server/fakepath/playlist.m3u8",
                                "hlsStatus": "ready",
                                "rtmp": {
                                    "foo": {
                                        "serverUrl": "rtmp://myfooserver/myfooapp",
                                        "streamName": "myfoostream"
                                    },
                                    "bar": {
                                        "serverUrl": "rtmp://mybarserver/mybarapp",
                                        "streamName": "mybarstream"
                                    }
                                }
                            }
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        options = {
            "layout": {"screenshareType": "verticalPresentation"},
            "maxDuration": 5400,
            "outputs": {
                "hls": {},
                "rtmp": [
                    {
                        "id": "foo",
                        "serverUrl": "rtmp://myfooserver/myfooapp",
                        "streamName": "myfoostream",
                    },
                    {
                        "id": "bar",
                        "serverUrl": "rtmp://mybarserver/mybarapp",
                        "streamName": "mybarstream",
                    },
                ],
            },
            "resolution": "640x480",
        }

        broadcast = self.opentok.start_broadcast(self.session_id, options)
        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        # non-deterministic json encoding. have to decode to test it properly
        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))

        expect(body).to(have_key("sessionId", self.session_id))
        expect(body).to(have_key("layout"))
        expect(body["layout"]).to(have_key("screenshareType"))
        expect(body["layout"]["screenshareType"]).to(equal("verticalPresentation"))
        expect(broadcast).to(be_an(Broadcast))
        expect(broadcast).to(
            have_property(u("id"), u("1748b7070a81464c9759c46ad10d3734"))
        )
        expect(broadcast).to(
            have_property(u("sessionId"), u("2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4"))
        )
        expect(broadcast).to(have_property(u("projectId"), 100))
        expect(broadcast).to(have_property(u("createdAt"), 1437676551000))
        expect(broadcast).to(have_property(u("updatedAt"), 1437676551000))
        expect(broadcast).to(have_property(u("resolution"), u("640x480")))
        expect(broadcast).to(have_property(u("status"), u("started")))
        expect(broadcast.broadcastUrls["hlsStatus"]).to(equal("ready"))
        expect(list(broadcast.broadcastUrls)).to(have_length(3))
        expect(list(broadcast.broadcastUrls["rtmp"])).to(have_length(2))

    @httpretty.activate
    def test_start_broadcast_audio_only(self):
        """
        Test start_broadcast() method
        """
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/broadcast").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "id": "1748b7070a81464c9759c46ad10d3734",
                            "sessionId": "2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4",
                            "projectId": 100,
                            "createdAt": 1437676551000,
                            "updatedAt": 1437676551000,
                            "resolution": "640x480",
                            "status": "started",
                            "hasAudio": true,
                            "hasVideo": false,
                            "maxBitrate": 1000000,
                            "maxDuration": 5400,
                            "hasAudio": true,
                            "hasVideo": false,
                            "broadcastUrls": {
                                "hls": "http://server/fakepath/playlist.m3u8",
                                "hlsStatus": "live",
                                "rtmp": {
                                    "foo": {
                                        "serverUrl": "rtmp://myfooserver/myfooapp",
                                        "streamName": "myfoostream"
                                    },
                                    "bar": {
                                        "serverUrl": "rtmp://mybarserver/mybarapp",
                                        "streamName": "mybarstream"
                                    }
                                }
                            }
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        options = {
            "layout": {
                "type": "custom",
                "stylesheet": "the layout stylesheet (only used with type == custom)",
            },
            "maxDuration": 5400,
            "hasAudio": True,
            "hasVideo": False,
            "outputs": {
                "hls": {},
                "rtmp": [
                    {
                        "id": "foo",
                        "serverUrl": "rtmp://myfooserver/myfooapp",
                        "streamName": "myfoostream",
                    },
                    {
                        "id": "bar",
                        "serverUrl": "rtmp://mybarserver/mybarapp",
                        "streamName": "mybarstream",
                    },
                ],
            },
            "resolution": "640x480",
        }

        broadcast = self.opentok.start_broadcast(self.session_id, options)
        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        # non-deterministic json encoding. have to decode to test it properly
        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))

        expect(body).to(have_key(u("layout")))
        expect(broadcast).to(be_an(Broadcast))
        expect(broadcast).to(
            have_property(u("id"), u("1748b7070a81464c9759c46ad10d3734"))
        )
        expect(broadcast).to(
            have_property(u("sessionId"), u("2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4"))
        )
        expect(broadcast).to(have_property(u("projectId"), 100))
        expect(broadcast).to(have_property(u("createdAt"), 1437676551000))
        expect(broadcast).to(have_property(u("updatedAt"), 1437676551000))
        expect(broadcast).to(have_property(u("hasAudio"), True))
        expect(broadcast).to(have_property(u("hasVideo"), False))
        expect(broadcast).to(have_property(u("resolution"), u("640x480")))
        expect(broadcast).to(have_property(u("status"), u("started")))
        expect(broadcast.broadcastUrls["hlsStatus"]).to(equal("live"))
        expect(list(broadcast.broadcastUrls)).to(have_length(3))
        expect(list(broadcast.broadcastUrls["rtmp"])).to(have_length(2))

    @httpretty.activate
    def test_start_broadcast_video_only(self):
        """
        Test start_broadcast() method
        """
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/broadcast").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "id": "1748b7070a81464c9759c46ad10d3734",
                            "sessionId": "2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4",
                            "projectId": 100,
                            "createdAt": 1437676551000,
                            "updatedAt": 1437676551000,
                            "resolution": "640x480",
                            "status": "started",
                            "hasAudio": false,
                            "hasVideo": true,
                            "maxBitrate": 1000000,
                            "maxDuration": 5400,
                            "hasAudio": false,
                            "hasVideo": true,
                            "broadcastUrls": {
                                "hls": "http://server/fakepath/playlist.m3u8",
                                "hlsStatus": "ready",
                                "rtmp": {
                                    "foo": {
                                        "serverUrl": "rtmp://myfooserver/myfooapp",
                                        "streamName": "myfoostream"
                                    },
                                    "bar": {
                                        "serverUrl": "rtmp://mybarserver/mybarapp",
                                        "streamName": "mybarstream"
                                    }
                                }
                            }
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        options = {
            "layout": {
                "type": "custom",
                "stylesheet": "the layout stylesheet (only used with type == custom)",
            },
            "maxDuration": 5400,
            "hasAudio": False,
            "hasVideo": True,
            "outputs": {
                "hls": {},
                "rtmp": [
                    {
                        "id": "foo",
                        "serverUrl": "rtmp://myfooserver/myfooapp",
                        "streamName": "myfoostream",
                    },
                    {
                        "id": "bar",
                        "serverUrl": "rtmp://mybarserver/mybarapp",
                        "streamName": "mybarstream",
                    },
                ],
            },
            "resolution": "640x480",
        }

        broadcast = self.opentok.start_broadcast(self.session_id, options)
        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        # non-deterministic json encoding. have to decode to test it properly
        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))

        expect(body).to(have_key(u("layout")))
        expect(broadcast).to(be_an(Broadcast))
        expect(broadcast).to(
            have_property(u("id"), u("1748b7070a81464c9759c46ad10d3734"))
        )
        expect(broadcast).to(
            have_property(u("sessionId"), u("2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4"))
        )
        expect(broadcast).to(have_property(u("projectId"), 100))
        expect(broadcast).to(have_property(u("createdAt"), 1437676551000))
        expect(broadcast).to(have_property(u("updatedAt"), 1437676551000))
        expect(broadcast).to(have_property(u("hasAudio"), False))
        expect(broadcast).to(have_property(u("hasVideo"), True))
        expect(broadcast).to(have_property(u("resolution"), u("640x480")))
        expect(broadcast).to(have_property(u("status"), u("started")))
        expect(broadcast.broadcastUrls["hlsStatus"]).to(equal("ready"))
        expect(list(broadcast.broadcastUrls)).to(have_length(3))
        expect(list(broadcast.broadcastUrls["rtmp"])).to(have_length(2))

    @httpretty.activate
    def test_start_broadcast_with_streammode_auto(self):
        url = f"https://api.opentok.com/v2/project/{self.api_key}/broadcast"

        httpretty.register_uri(
            httpretty.POST,
            url,
            responses=[
                httpretty.Response(
                    body=json.dumps({"stream_mode": "auto"}),
                    content_type="application/json",
                    status=200,
                )
            ],
        )

        broadcast = self.opentok.start_broadcast(
            session_id=self.session_id, options={"outputs": {"hls": {}}}
        )
        broadcast.stream_mode.should.equal(BroadcastStreamModes.auto)

    @httpretty.activate
    def test_add_broadcast_stream(self):
        broadcast_id = "BROADCASTID"
        url = f"https://api.opentok.com/v2/project/{self.api_key}/broadcast/{broadcast_id}/streams"
        httpretty.register_uri(
            httpretty.PATCH, url, responses=[httpretty.Response(body=u(""), status=204)]
        )

        response = self.opentok.add_broadcast_stream(
            broadcast_id=broadcast_id, stream_id=self.stream1
        )
        assert response == None

    @httpretty.activate
    def test_remove_broadcast_stream(self):
        broadcast_id = "BROADCASTID"
        url = f"https://api.opentok.com/v2/project/{self.api_key}/broadcast/{broadcast_id}/streams"
        httpretty.register_uri(
            httpretty.PATCH, url, responses=[httpretty.Response(body=u(""), status=204)]
        )

        response = self.opentok.add_broadcast_stream(
            broadcast_id=broadcast_id, stream_id=self.stream1
        )
        assert response == None

    @httpretty.activate
    def test_update_broadcast_auto(self):
        broadcast_id = u("BROADCASTID")
        url = f"https://api.opentok.com/v2/project/{self.api_key}/broadcast/{broadcast_id}/streams"

        payload = {
            "hasAudio": True,
            "hasVideo": True,
            "addStream": [self.stream1, self.stream2],
        }

        httpretty.register_uri(
            httpretty.PATCH,
            url,
            responses=[
                httpretty.Response(
                    body=json.dumps(payload),
                    content_type="application/json",
                    status=200,
                )
            ],
        )

        response = requests.patch(url)

        response.status_code.should.equal(200)
        response.json().should.equal(
            {
                "hasAudio": True,
                "hasVideo": True,
                "addStream": [self.stream1, self.stream2],
            }
        )

        response.headers["Content-Type"].should.equal("application/json")

    @httpretty.activate
    def test_update_broadcast_manual(self):
        broadcast_id = u("BROADCASTID")
        url = f"https://api.opentok.com/v2/project/{self.api_key}/broadcast/{broadcast_id}/streams"

        payload = {"hasAudio": True, "hasVideo": True, "addStream": [self.stream1]}

        httpretty.register_uri(
            httpretty.PATCH,
            url,
            responses=[
                httpretty.Response(
                    body=json.dumps(payload),
                    content_type="application/json",
                    status=200,
                )
            ],
        )

        response = requests.patch(url)

        response.status_code.should.equal(200)
        response.json().should.equal(
            {"hasAudio": True, "hasVideo": True, "addStream": [self.stream1]}
        )

        response.headers["Content-Type"].should.equal("application/json")

    @httpretty.activate
    def test_stop_broadcast(self):
        """
        Test stop_broadcast() method
        """
        broadcast_id = u("1748b7070a81464c9759c46ad10d3734")

        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/broadcast/{1}/stop").format(
                self.api_key, broadcast_id
            ),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "id": "1748b7070a81464c9759c46ad10d3734",
                            "sessionId": "2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4",
                            "projectId": 100,
                            "createdAt": 1437676551000,
                            "updatedAt": 1437676551000,
                            "resolution": "640x480",
                            "broadcastUrls": null
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        broadcast = self.opentok.stop_broadcast(broadcast_id)
        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        expect(broadcast).to(be_an(Broadcast))
        expect(broadcast).to(
            have_property(u("id"), u("1748b7070a81464c9759c46ad10d3734"))
        )
        expect(broadcast).to(
            have_property(u("sessionId"), u("2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4"))
        )
        expect(broadcast).to(have_property(u("projectId"), 100))
        expect(broadcast).to(have_property(u("createdAt"), 1437676551000))
        expect(broadcast).to(have_property(u("updatedAt"), 1437676551000))
        expect(broadcast).to(have_property(u("resolution"), u("640x480")))

    @httpretty.activate
    def test_get_broadcast(self):
        """
        Test get_broadcast() method
        """
        broadcast_id = u("1748b707-0a81-464c-9759-c46ad10d3734")

        httpretty.register_uri(
            httpretty.GET,
            u("https://api.opentok.com/v2/project/{0}/broadcast/{1}").format(
                self.api_key, broadcast_id
            ),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "id": "1748b707-0a81-464c-9759-c46ad10d3734",
                            "sessionId": "2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4",
                            "projectId": 100,
                            "createdAt": 1437676551000,
                            "updatedAt": 1437676551000,
                            "resolution": "640x480",
                            "broadcastUrls": {
                                "hls": "http://server/fakepath/playlist.m3u8",
                                "hlsStatus": "live",
                                "rtmp": {
                                    "foo": {
                                        "serverUrl": "rtmp://myfooserver/myfooapp",
                                        "streamName": "myfoostream",
                                        "status": "live"
                                    },
                                    "bar": {
                                        "serverUrl": "rtmp://mybarserver/mybarapp",
                                        "streamName": "mybarstream",
                                        "status": "live"
                                    }
                                }
                            },
                            "status": "started"
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        broadcast = self.opentok.get_broadcast(broadcast_id)
        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        expect(broadcast).to(be_an(Broadcast))
        expect(broadcast).to(have_property(u("id"), broadcast_id))
        expect(broadcast).to(
            have_property(u("sessionId"), u("2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4"))
        )
        expect(broadcast).to(have_property(u("projectId"), 100))
        expect(broadcast).to(have_property(u("createdAt"), 1437676551000))
        expect(broadcast).to(have_property(u("updatedAt"), 1437676551000))
        expect(broadcast).to(have_property(u("resolution"), u("640x480")))
        expect(broadcast).to(have_property(u("status"), u("started")))
        expect(broadcast.broadcastUrls["hlsStatus"]).to(equal("live"))
        expect(list(broadcast.broadcastUrls)).to(have_length(3))
        expect(list(broadcast.broadcastUrls["rtmp"])).to(have_length(2))

    @httpretty.activate
    def test_set_broadcast_layout(self):
        """Test set_broadcast_layout() functionality"""
        broadcast_id = u("1748b707-0a81-464c-9759-c46ad10d3734")

        httpretty.register_uri(
            httpretty.PUT,
            u("https://api.opentok.com/v2/project/{0}/broadcast/{1}/layout").format(
                self.api_key, broadcast_id
            ),
            status=200,
            content_type=u("application/json"),
        )

        self.opentok.set_broadcast_layout(broadcast_id, "horizontalPresentation")

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )

    @httpretty.activate
    def test_set_broadcast_layout_with_screenshare_type(self):
        """Test set_broadcast_layout() functionality"""
        broadcast_id = u("1748b707-0a81-464c-9759-c46ad10d3734")

        httpretty.register_uri(
            httpretty.PUT,
            u("https://api.opentok.com/v2/project/{0}/broadcast/{1}/layout").format(
                self.api_key, broadcast_id
            ),
            status=200,
            content_type=u("application/json"),
        )

        self.opentok.set_broadcast_layout(
            broadcast_id, "bestFit", screenshare_type="horizontalPresentation"
        )

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))

        expect(body).to(have_key(u("type"), u("bestFit")))
        expect(body).to_not(have_key(u("stylesheet")))
        expect(body).to(have_key(u("screenshareType"), u("horizontalPresentation")))

    @httpretty.activate
    def test_set_custom_broadcast_layout(self):
        """Test set a custom broadcast layout specifying the 'stylesheet' parameter"""
        broadcast_id = u("1748b707-0a81-464c-9759-c46ad10d3734")

        httpretty.register_uri(
            httpretty.PUT,
            u("https://api.opentok.com/v2/project/{0}/broadcast/{1}/layout").format(
                self.api_key, broadcast_id
            ),
            status=200,
            content_type=u("application/json"),
        )

        self.opentok.set_broadcast_layout(
            broadcast_id,
            "custom",
            "stream.instructor {position: absolute; width: 100%;  height:50%;}",
        )

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )

    @httpretty.activate
    def test_set_broadcast_layout_throws_exception(self):
        """Test invalid request in set broadcast layout"""
        broadcast_id = u("1748b707-0a81-464c-9759-c46ad10d3734")

        httpretty.register_uri(
            httpretty.PUT,
            u("https://api.opentok.com/v2/project/{0}/broadcast/{1}/layout").format(
                self.api_key, broadcast_id
            ),
            status=400,
            content_type=u("application/json"),
        )

        self.assertRaises(
            BroadcastError,
            self.opentok.set_broadcast_layout,
            broadcast_id,
            "horizontalPresentation",
        )

    def test_broadcast_hls_mutually_exclusive_options_error(self):
        """
        Test invalid options in start_broadcast() method raises a BroadcastHLSOptionsError.
        """

        options = {
            "layout": {
                "type": "custom",
                "stylesheet": "the layout stylesheet (only used with type == custom)",
            },
            "maxDuration": 5400,
            "outputs": {
                "hls": {"lowLatency": True, "dvr": True},
                "rtmp": [
                    {
                        "id": "foo",
                        "serverUrl": "rtmp://myfooserver/myfooapp",
                        "streamName": "myfoostream",
                    },
                    {
                        "id": "bar",
                        "serverUrl": "rtmp://mybarserver/mybarapp",
                        "streamName": "mybarstream",
                    },
                ],
            },
            "resolution": "640x480",
        }

        with self.assertRaises(BroadcastHLSOptionsError):
            self.opentok.start_broadcast(self.session_id, options)



import unittest
from six import u
from expects import *

from opentok import Client, __version__


class GetterSetterTest(unittest.TestCase):
    def setUp(self):
        self.api_key = u("123456")
        self.api_secret = u("1234567890abcdef1234567890abcdef1234567890")
        self.session_id = u(
            "1_MX4xMjM0NTZ-flNhdCBNYXIgMTUgMTQ6NDI6MjMgUERUIDIwMTR-MC40OTAxMzAyNX4"
        )

        self.opentok = Client(self.api_key, self.api_secret)

    def test_getset_jwt_livetime(self):
        livetime = self.opentok.jwt_livetime
        self.opentok.jwt_livetime = 5
        assert self.opentok.jwt_livetime is not livetime


import json


class Captions:
    """Represents information about a captioning session."""

    def __init__(self, kwargs):
        self.captions_id = kwargs.get("captionsId")

    def json(self):
        """Returns a JSON representation of the captioning session information."""
        return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True, indent=4)



from time import time
from jwt import decode
import pytest
import unittest
from six import u, b
from urllib.parse import parse_qs
from expects import *
import httpretty
from .validate_jwt import validate_jwt_header
import platform

from opentok import (
    Client,
    Session,
    MediaModes,
    ArchiveModes,
    OpenTokException,
    __version__,
)


class OpenTokSessionCreationTest(unittest.TestCase):
    def setUp(self):
        self.api_key = u("123456")
        self.api_secret = u("1234567890abcdef1234567890abcdef1234567890")
        self.opentok = Client(self.api_key, self.api_secret)

    @httpretty.activate
    def test_create_default_session(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/session/create"),
            body='[{"session_id":"1_MX40NzY2NTk3MX5-MTczOTUzODg5NDk1OX5ROW1jWEcxUXJOM1RJWXU4eStwcHgvZ3N-UH5-","project_id":"47665971","partner_id":"47665971","create_dt":"Fri Feb 14 05:14:54 PST 2025","session_status":null,"status_invalid":null,"media_server_hostname":null,"messaging_server_url":null,"messaging_url":null,"symphony_address":null,"properties":null,"ice_server":null,"session_segment_id":"9c5fdd5b-7f5b-408f-8633-db95d0054fad","ice_servers":null,"ice_credential_expiration":86100}]',
            status=200,
            content_type=u("application/json"),
        )

        session = self.opentok.create_session()

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            equal(
                u("OpenTok-Python-SDK/")
                + __version__
                + " python/"
                + platform.python_version()
            )
        )
        body = parse_qs(httpretty.last_request().body)
        expect(body).to(have_key(b("p2p.preference"), [b("enabled")]))
        expect(body).to(have_key(b("archiveMode"), [b("manual")]))
        expect(session).to(be_a(Session))
        expect(session).to(
            have_property(
                u("session_id"),
                u(
                    "1_MX40NzY2NTk3MX5-MTczOTUzODg5NDk1OX5ROW1jWEcxUXJOM1RJWXU4eStwcHgvZ3N-UH5-"
                ),
            )
        )
        expect(session).to(have_property(u("media_mode"), MediaModes.relayed))
        expect(session).to(have_property(u("location"), None))
        expect(session).to(have_property(u("e2ee"), False))

    @httpretty.activate
    def test_create_default_vonage_session(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://video.api.vonage.com/session/create"),
            body="""
                [
                    {
                        "session_id": "1_MX4yOWY3NjBmOC03Y2UxLTQ2YzktYWRlMy1mMmRlZGVlNGVkNWZ-fjE3MjY0NjI1ODg2NDd-MTF4TGExYmJoelBlR1FHbVhzbWd4STBrfn5-",
                        "project_id": "29f760f8-7ce1-46c9-ade3-f2dedee4ed5f",
                        "partner_id": "29f760f8-7ce1-46c9-ade3-f2dedee4ed5f",
                        "create_dt": "Sun Sep 15 21:56:28 PDT 2024",
                        "session_status": null,
                        "status_invalid": null,
                        "media_server_hostname": null,
                        "messaging_server_url": null,
                        "messaging_url": null,
                        "symphony_address": null,
                        "properties": null,
                        "ice_server": null,
                        "session_segment_id": "35308566-4012-4c1e-90f7-cc15b5a390fe",
                        "ice_servers": null,
                        "ice_credential_expiration": 86100
                    }
                ]""",
            status=200,
            content_type="application/json",
        )

        self.api_secret = './tests/fake_data/dummy_private_key.txt'
        vonage_wrapper = Client(self.api_key, self.api_secret)
        session = vonage_wrapper.create_session()

        public_key = ""
        with open('./tests/fake_data/dummy_public_key.txt', 'r') as file:
            public_key = file.read()

        decoded_jwt = decode(
            httpretty.last_request().headers[u("Authorization")].split(None, 1)[1],
            public_key,
            algorithms=["RS256"],
        )

        expect(decoded_jwt["application_id"]).to(equal(vonage_wrapper.api_key))
        expect(decoded_jwt["ist"]).to(equal("project"))
        expect(decoded_jwt["exp"]).to(be_above(time()))

        expect(httpretty.last_request().headers[u("user-agent")]).to(
            equal(
                u("OpenTok-Python-SDK/")
                + __version__
                + " python/"
                + platform.python_version()
                + " OpenTok-With-Vonage-API-Backend"
            )
        )
        body = parse_qs(httpretty.last_request().body)
        expect(body).to(have_key(b("p2p.preference"), [b("enabled")]))
        expect(body).to(have_key(b("archiveMode"), [b("manual")]))
        expect(session).to(be_a(Session))
        expect(session).to(
            have_property(
                u("session_id"),
                u(
                    "1_MX4yOWY3NjBmOC03Y2UxLTQ2YzktYWRlMy1mMmRlZGVlNGVkNWZ-fjE3MjY0NjI1ODg2NDd-MTF4TGExYmJoelBlR1FHbVhzbWd4STBrfn5-"
                ),
            )
        )
        expect(session).to(have_property(u("media_mode"), MediaModes.relayed))
        expect(session).to(have_property(u("location"), None))
        expect(session).to(have_property(u("e2ee"), False))

    @httpretty.activate
    def test_create_routed_session(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/session/create"),
            body='[{"session_id":"1_MX40NzY2NTk3MX5-MTczOTUzODg5NDk1OX5ROW1jWEcxUXJOM1RJWXU4eStwcHgvZ3N-UH5-","project_id":"47665971","partner_id":"47665971","create_dt":"Fri Feb 14 05:14:54 PST 2025","session_status":null,"status_invalid":null,"media_server_hostname":null,"messaging_server_url":null,"messaging_url":null,"symphony_address":null,"properties":null,"ice_server":null,"session_segment_id":"9c5fdd5b-7f5b-408f-8633-db95d0054fad","ice_servers":null,"ice_credential_expiration":86100}]',
            status=200,
            content_type=u("application/json"),
        )

        session = self.opentok.create_session(media_mode=MediaModes.routed)

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        body = parse_qs(httpretty.last_request().body)
        expect(body).to(have_key(b("p2p.preference"), [b("disabled")]))
        expect(body).to(have_key(b("archiveMode"), [b("manual")]))
        expect(session).to(be_a(Session))
        expect(session).to(
            have_property(
                u("session_id"),
                u(
                    "1_MX40NzY2NTk3MX5-MTczOTUzODg5NDk1OX5ROW1jWEcxUXJOM1RJWXU4eStwcHgvZ3N-UH5-"
                ),
            )
        )
        expect(session).to(have_property(u("media_mode"), MediaModes.routed))
        expect(session).to(have_property(u("location"), None))
        expect(session).to(have_property(u("e2ee"), False))

    @httpretty.activate
    def test_failure_create_routed_session(self):
        # Session creation fails when server doesn't returns a XML
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/session/create"),
            body=u(
                '<html><head><meta charset="UTF-8"></head><body>Page not found</body></html>'
            ),
            status=200,
            content_type=u("text/xml"),
        )

        with pytest.raises(OpenTokException):
            self.opentok.create_session(media_mode=MediaModes.routed)

    @httpretty.activate
    def test_create_session_with_location_hint(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/session/create"),
            body=u(
                '<?xml version="1.0" encoding="UTF-8" standalone="yes"?><sessions><Session><session_id>1_MX4xMjM0NTZ-fk1vbiBNYXIgMTcgMDA6NDE6MzEgUERUIDIwMTR-MC42ODM3ODk1MzQ0OTQyODA4fg</session_id><partner_id>123456</partner_id><create_dt>Mon Mar 17 00:41:31 PDT 2014</create_dt></Session></sessions>'
            ),
            status=200,
            content_type=u("text/xml"),
        )

        session = self.opentok.create_session(location="12.34.56.78")

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        # ordering of keys is non-deterministic, must parse the body to see if it is correct
        body = parse_qs(httpretty.last_request().body)
        expect(body).to(have_key(b("location"), [b("12.34.56.78")]))
        expect(body).to(have_key(b("p2p.preference"), [b("enabled")]))
        expect(session).to(be_a(Session))
        expect(session).to(
            have_property(
                u("session_id"),
                u(
                    "1_MX4xMjM0NTZ-fk1vbiBNYXIgMTcgMDA6NDE6MzEgUERUIDIwMTR-MC42ODM3ODk1MzQ0OTQyODA4fg"
                ),
            )
        )
        expect(session).to(have_property(u("media_mode"), MediaModes.relayed))
        expect(session).to(have_property(u("location"), u("12.34.56.78")))
        expect(session).to(have_property(u("e2ee"), False))

    @httpretty.activate
    def test_create_routed_session_with_location_hint(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/session/create"),
            body=u(
                '<?xml version="1.0" encoding="UTF-8" standalone="yes"?><sessions><Session><session_id>1_MX4xMjM0NTZ-fk1vbiBNYXIgMTcgMDA6NDE6MzEgUERUIDIwMTR-MC42ODM3ODk1MzQ0OTQyODA4fg</session_id><partner_id>123456</partner_id><create_dt>Mon Mar 17 00:41:31 PDT 2014</create_dt></Session></sessions>'
            ),
            status=200,
            content_type=u("text/xml"),
        )

        session = self.opentok.create_session(
            location="12.34.56.78", media_mode=MediaModes.routed
        )

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        # ordering of keys is non-deterministic, must parse the body to see if it is correct
        body = parse_qs(httpretty.last_request().body)
        expect(body).to(have_key(b("location"), [b("12.34.56.78")]))
        expect(body).to(have_key(b("p2p.preference"), [b("disabled")]))
        expect(session).to(be_a(Session))
        expect(session).to(
            have_property(
                u("session_id"),
                u(
                    "1_MX4xMjM0NTZ-fk1vbiBNYXIgMTcgMDA6NDE6MzEgUERUIDIwMTR-MC42ODM3ODk1MzQ0OTQyODA4fg"
                ),
            )
        )
        expect(session).to(have_property(u("media_mode"), MediaModes.routed))
        expect(session).to(have_property(u("location"), u("12.34.56.78")))
        expect(session).to(have_property(u("e2ee"), False))

    @httpretty.activate
    def test_create_manual_archive_mode_session(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/session/create"),
            body=u(
                '<?xml version="1.0" encoding="UTF-8" standalone="yes"?><sessions><Session><session_id>1_MX4xMjM0NTZ-fk1vbiBNYXIgMTcgMDA6NDE6MzEgUERUIDIwMTR-MC42ODM3ODk1MzQ0OTQyODA4fg</session_id><partner_id>123456</partner_id><create_dt>Mon Mar 17 00:41:31 PDT 2014</create_dt></Session></sessions>'
            ),
            status=200,
            content_type=u("text/xml"),
        )

        session = self.opentok.create_session(
            media_mode=MediaModes.routed, archive_mode=ArchiveModes.manual
        )

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        body = parse_qs(httpretty.last_request().body)
        expect(body).to(have_key(b("p2p.preference"), [b("disabled")]))
        expect(body).to(have_key(b("archiveMode"), [b("manual")]))
        expect(session).to(be_a(Session))
        expect(session).to(
            have_property(
                u("session_id"),
                u(
                    "1_MX4xMjM0NTZ-fk1vbiBNYXIgMTcgMDA6NDE6MzEgUERUIDIwMTR-MC42ODM3ODk1MzQ0OTQyODA4fg"
                ),
            )
        )
        expect(session).to(have_property(u("media_mode"), MediaModes.routed))
        expect(session).to(have_property(u("archive_mode"), ArchiveModes.manual))
        expect(session).to(have_property(u("e2ee"), False))

    @httpretty.activate
    def test_create_always_archive_mode_session(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/session/create"),
            body=u(
                '<?xml version="1.0" encoding="UTF-8" standalone="yes"?><sessions><Session><session_id>1_MX4xMjM0NTZ-fk1vbiBNYXIgMTcgMDA6NDE6MzEgUERUIDIwMTR-MC42ODM3ODk1MzQ0OTQyODA4fg</session_id><partner_id>123456</partner_id><create_dt>Mon Mar 17 00:41:31 PDT 2014</create_dt></Session></sessions>'
            ),
            status=200,
            content_type=u("text/xml"),
        )

        session = self.opentok.create_session(
            media_mode=MediaModes.routed,
            archive_mode=ArchiveModes.always,
            archive_name="test_opentok_archive",
            archive_resolution="1920x1080",
        )

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        body = parse_qs(httpretty.last_request().body)
        expect(body).to(have_key(b("p2p.preference"), [b("disabled")]))
        expect(body).to(have_key(b("archiveMode"), [b("always")]))
        expect(body).to(have_key(b("archiveName"), [b("test_opentok_archive")]))
        expect(body).to(have_key(b("archiveResolution"), [b("1920x1080")]))
        expect(session).to(be_a(Session))
        expect(session).to(
            have_property(
                u("session_id"),
                u(
                    "1_MX4xMjM0NTZ-fk1vbiBNYXIgMTcgMDA6NDE6MzEgUERUIDIwMTR-MC42ODM3ODk1MzQ0OTQyODA4fg"
                ),
            )
        )
        expect(session).to(have_property(u("media_mode"), MediaModes.routed))
        expect(session).to(have_property(u("archive_mode"), ArchiveModes.always))
        expect(session).to(have_property(u("e2ee"), False))

    def test_complains_about_always_archive_mode_and_relayed_session(self):
        with pytest.raises(OpenTokException):
            self.opentok.create_session(
                media_mode=MediaModes.relayed, archive_mode=ArchiveModes.always
            )

    def test_auto_archive_errors(self):
        with pytest.raises(OpenTokException):
            self.opentok.create_session(
                media_mode=MediaModes.routed,
                archive_mode=ArchiveModes.manual,
                archive_name="my_archive",
            )

        with pytest.raises(OpenTokException):
            self.opentok.create_session(
                media_mode=MediaModes.routed,
                archive_mode=ArchiveModes.manual,
                archive_resolution="640x480",
            )

        with pytest.raises(OpenTokException):
            self.opentok.create_session(
                media_mode=MediaModes.routed,
                archive_mode=ArchiveModes.always,
                archive_name="my_incredibly_long_name_that_is_definitely_going_to_be_over_the_80_character_limit_we_currently_impose",
            )

        with pytest.raises(OpenTokException):
            self.opentok.create_session(
                media_mode=MediaModes.routed,
                archive_mode=ArchiveModes.always,
                archive_resolution="10x10",
            )

    @httpretty.activate
    def test_create_session_with_e2ee(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/session/create"),
            body=u(
                '<?xml version="1.0" encoding="UTF-8" standalone="yes"?><sessions><Session><session_id>1_MX4xMjM0NTZ-fk1vbiBNYXIgMTcgMDA6NDE6MzEgUERUIDIwMTR-MC42ODM3ODk1MzQ0OTQyODA4fg</session_id><partner_id>123456</partner_id><create_dt>Mon Mar 17 00:41:31 PDT 2014</create_dt></Session></sessions>'
            ),
            status=200,
            content_type=u("text/xml"),
        )

        session = self.opentok.create_session(e2ee=True)

        body = parse_qs(httpretty.last_request().body)
        expect(body).to(have_key(b("e2ee"), [b"true"]))
        expect(session).to(be_a(Session))
        expect(session).to(
            have_property(
                u("session_id"),
                u(
                    "1_MX4xMjM0NTZ-fk1vbiBNYXIgMTcgMDA6NDE6MzEgUERUIDIwMTR-MC42ODM3ODk1MzQ0OTQyODA4fg"
                ),
            )
        )
        expect(session).to(have_property(u("e2ee"), True))

    # TODO: all the cases that throw exceptions
    # TODO: custom api_url requests



import unittest
import textwrap
import httpretty
import json
from sure import expect

from six import u, PY2, PY3
from expects import *
from opentok import Client, WebSocketAudioConnection, __version__
from opentok.exceptions import InvalidWebSocketOptionsError, InvalidMediaModeError
from .validate_jwt import validate_jwt_header


class OpenTokAudioConnectorLiteTest(unittest.TestCase):
    def setUp(self):
        self.api_key = u("123456")
        self.api_secret = u("1234567890abcdef1234567890abcdef1234567890")
        self.opentok = Client(self.api_key, self.api_secret)
        self.session_id = u("2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4")
        self.token = u("1234-5678-9012")
        self.response_body = textwrap.dedent(
            u(
                """ \
                    {
                        "id": "b0a5a8c7-dc38-459f-a48d-a7f2008da853",
                        "connectionId": "e9f8c166-6c67-440d-994a-04fb6dfed007"
                    }
                """
            )
        )

    @httpretty.activate
    def test_connect_audio_to_websocket(self):
        httpretty.register_uri(
            httpretty.POST,
            u(f"https://api.opentok.com/v2/project/{self.api_key}/connect"),
            body=self.response_body,
            status=200,
            content_type=u("application/json"),
        )

        websocket_options = {"uri": "wss://service.com/ws-endpoint"}

        websocket_audio_connection = self.opentok.connect_audio_to_websocket(
            self.session_id, self.token, websocket_options
        )

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(contain(u("OpenTok-Python-SDK/") + __version__))
        expect(httpretty.last_request().headers[u("content-type")]).to(equal(u("application/json")))
        # non-deterministic json encoding. have to decode to test it properly
        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))

        expect(body).to(have_key(u("token")))
        expect(websocket_audio_connection).to(be_a(WebSocketAudioConnection))
        expect(websocket_audio_connection).to(have_property(u("id"), u("b0a5a8c7-dc38-459f-a48d-a7f2008da853")))
        expect(websocket_audio_connection).to(
            have_property(u("connectionId"), u("e9f8c166-6c67-440d-994a-04fb6dfed007"))
        )

    @httpretty.activate
    def test_connect_audio_to_websocket_custom_options(self):
        httpretty.register_uri(
            httpretty.POST,
            u(f"https://api.opentok.com/v2/project/{self.api_key}/connect"),
            body=self.response_body,
            status=200,
            content_type=u("application/json"),
        )

        websocket_options = {
            "uri": "wss://service.com/ws-endpoint",
            "streams": ["stream-id-1", "stream-id-2"],
            "headers": {"WebSocketHeader": "Sent via Audio Connector API"},
        }

        websocket_audio_connection = self.opentok.connect_audio_to_websocket(
            self.session_id, self.token, websocket_options
        )

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(contain(u("OpenTok-Python-SDK/") + __version__))
        expect(httpretty.last_request().headers[u("content-type")]).to(equal(u("application/json")))
        # non-deterministic json encoding. have to decode to test it properly
        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))

        expect(body).to(have_key(u("token")))
        expect(websocket_audio_connection).to(be_a(WebSocketAudioConnection))
        expect(websocket_audio_connection).to(have_property(u("id"), u("b0a5a8c7-dc38-459f-a48d-a7f2008da853")))
        expect(websocket_audio_connection).to(
            have_property(u("connectionId"), u("e9f8c166-6c67-440d-994a-04fb6dfed007"))
        )

    @httpretty.activate
    def test_connect_audio_to_websocket_media_mode_error(self):
        httpretty.register_uri(
            httpretty.POST,
            u(f"https://api.opentok.com/v2/project/{self.api_key}/connect"),
            body={},
            status=409,
            content_type=u("application/json"),
        )

        websocket_options = {"uri": "wss://service.com/ws-endpoint"}

        session_id = "session-where-mediaMode=relayed-was-selected"

        with self.assertRaises(InvalidMediaModeError) as context:
            self.opentok.connect_audio_to_websocket(session_id, self.token, websocket_options)
        self.assertTrue(
            "Only routed sessions are allowed to initiate Audio Connector WebSocket connections."
            in str(context.exception)
        )

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(contain(u("OpenTok-Python-SDK/") + __version__))
        expect(httpretty.last_request().headers[u("content-type")]).to(equal(u("application/json")))

    def test_connect_audio_to_websocket_invalid_options_type_error(self):
        websocket_options = "wss://service.com/ws-endpoint"
        with self.assertRaises(InvalidWebSocketOptionsError) as context:
            self.opentok.connect_audio_to_websocket(self.session_id, self.token, websocket_options)
        self.assertTrue("Must pass WebSocket options as a dictionary." in str(context.exception))

    def test_connect_audio_to_websocket_missing_uri_error(self):
        websocket_options = {}
        with self.assertRaises(InvalidWebSocketOptionsError) as context:
            self.opentok.connect_audio_to_websocket(self.session_id, self.token, websocket_options)
        self.assertTrue("Provide a WebSocket URI." in str(context.exception))



import unittest
from six import text_type, u

from opentok import Client, Session, Roles, MediaModes
from .helpers import token_decoder


class SessionTest(unittest.TestCase):
    def setUp(self):
        self.api_key = u("123456")
        self.api_secret = u("1234567890abcdef1234567890abcdef1234567890")
        self.session_id = u(
            "1_MX4xMjM0NTZ-flNhdCBNYXIgMTUgMTQ6NDI6MjMgUERUIDIwMTR-MC40OTAxMzAyNX4"
        )
        self.opentok = Client(self.api_key, self.api_secret)

    def test_generate_token(self):
        session = Session(
            self.opentok, self.session_id, media_mode=MediaModes.routed, location=None
        )
        token = session.generate_token()
        assert isinstance(token, text_type)
        assert token_decoder(token, self.api_secret)[u("session_id")] == self.session_id

    def test_generate_role_token(self):
        session = Session(
            self.opentok, self.session_id, media_mode=MediaModes.routed, location=None
        )
        token = session.generate_token(role=Roles.moderator)
        assert isinstance(token, text_type)
        assert token_decoder(token, self.api_secret)[u("session_id")] == self.session_id
        assert token_decoder(token, self.api_secret)[u("role")] == u("moderator")



import unittest
from six import u, PY2, PY3
from expects import *
import httpretty
from sure import expect
import textwrap
import datetime
import pytz
from .validate_jwt import validate_jwt_header

from opentok import Client, Archive, __version__, OutputModes


class OpenTokArchiveTest(unittest.TestCase):
    def setUp(self):
        self.api_key = u("123456")
        self.api_secret = u("1234567890abcdef1234567890abcdef1234567890")
        self.stream1 = "kjhk-09898-kjn"
        self.stream2 = "jnn-99kjk-88734r"
        self.opentok = Client(self.api_key, self.api_secret)

    @httpretty.activate
    def test_stop_archive(self):
        archive_id = u("ARCHIVEID")
        archive = Archive(
            self.opentok,
            {
                u("createdAt"): 1395183243556,
                u("duration"): 0,
                u("id"): archive_id,
                u("name"): u(""),
                u("partnerId"): 123456,
                u("reason"): u(""),
                u("sessionId"): u("SESSIONID"),
                u("size"): 0,
                u("status"): u("started"),
                u("hasAudio"): True,
                u("hasVideo"): True,
                u("outputMode"): OutputModes.composed.value,
                u("url"): None,
                u("maxBitrate"): 2000000,
            },
        )
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/archive/{1}/stop").format(
                self.api_key, archive_id
            ),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "createdAt" : 1395183243556,
                            "duration" : 0,
                            "id" : "ARCHIVEID",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 0,
                            "status" : "stopped",
                            "hasAudio": true,
                            "hasVideo": false,
                            "outputMode": "composed",
                            "url" : null,
                            "maxBitrate": 2000000
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive.stop()

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        expect(archive).to(be_an(Archive))
        expect(archive).to(have_property(u("id"), archive_id))
        expect(archive).to(have_property(u("name"), u("")))
        expect(archive).to(have_property(u("status"), u("stopped")))
        expect(archive).to(have_property(u("session_id"), u("SESSIONID")))
        expect(archive).to(have_property(u("partner_id"), 123456))
        if PY2:
            created_at = datetime.datetime.fromtimestamp(1395183243, pytz.UTC)
        if PY3:
            created_at = datetime.datetime.fromtimestamp(
                1395183243, datetime.timezone.utc
            )
        expect(archive).to(have_property(u("created_at"), created_at))
        expect(archive).to(have_property(u("size"), 0))
        expect(archive).to(have_property(u("duration"), 0))
        expect(archive).to(have_property(u("has_audio"), True))
        expect(archive).to(have_property(u("has_video"), False))
        expect(archive).to(have_property(u("output_mode"), OutputModes.composed))
        expect(archive).to(have_property(u("url"), None))
        expect(archive).to(have_property(u("max_bitrate"), 2000000))

    @httpretty.activate
    def test_add_archive_stream(self):
        archive_id = u("ARCHIVEID")
        url = f"https://api.opentok.com/v2/project/{self.api_key}/archive/{archive_id}/streams"
        httpretty.register_uri(
            httpretty.PATCH, url, responses=[httpretty.Response(body=u(""), status=204)]
        )

        response = self.opentok.add_archive_stream(
            archive_id=archive_id, stream_id=self.stream1
        )
        assert response == None

    @httpretty.activate
    def test_remove_archive_stream(self):
        archive_id = u("ARCHIVEID")
        url = f"https://api.opentok.com/v2/project/{self.api_key}/archive/{archive_id}/streams"
        httpretty.register_uri(
            httpretty.PATCH, url, responses=[httpretty.Response(body=u(""), status=204)]
        )

        response = self.opentok.remove_archive_stream(
            archive_id=archive_id, stream_id=self.stream1
        )
        assert response == None

    @httpretty.activate
    def test_delete_archive(self):
        archive_id = u("ARCHIVEID")
        archive = Archive(
            self.opentok,
            {
                u("createdAt"): 1395183243556,
                u("duration"): 0,
                u("id"): archive_id,
                u("name"): u(""),
                u("partnerId"): 123456,
                u("reason"): u(""),
                u("sessionId"): u("SESSIONID"),
                u("size"): 0,
                u("status"): u("available"),
                u("hasAudio"): True,
                u("hasVideo"): True,
                u("outputMode"): OutputModes.composed.value,
                u("url"): None,
                u("maxBitrate"): 2000000,
            },
        )
        httpretty.register_uri(
            httpretty.DELETE,
            u("https://api.opentok.com/v2/project/{0}/archive/{1}").format(
                self.api_key, archive_id
            ),
            body=u(""),
            status=204,
        )

        archive.delete()

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        # TODO: test that the object is invalidated



import pytest
import unittest
from six import text_type, u, b, PY2, PY3
from expects import *
import httpretty
from sure import expect
import textwrap
import json
import datetime
import pytz
import requests

from .validate_jwt import validate_jwt_header

from opentok import (
    Client,
    Archive,
    ArchiveList,
    OutputModes,
    OpenTokException,
    __version__,
    ArchiveError,
)


class OpenTokArchiveApiTest(unittest.TestCase):
    def setUp(self):
        self.api_key = u("123456")
        self.api_secret = u("1234567890abcdef1234567890abcdef1234567890")
        self.session_id = u("SESSIONID")
        self.opentok = Client(self.api_key, self.api_secret)

    @httpretty.activate
    def test_start_archive(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/archive").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "createdAt" : 1395183243556,
                            "duration" : 0,
                            "id" : "30b3ebf1-ba36-4f5b-8def-6f70d9986fe9",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 0,
                            "status" : "started",
                            "hasAudio": true,
                            "hasVideo": true,
                            "outputMode": "composed",
                            "url" : null,
                            "maxBitrate": 2000000
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive = self.opentok.start_archive(self.session_id)

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        # non-deterministic json encoding. have to decode to test it properly
        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))
        expect(body).to(have_key(u("name"), None))
        expect(body).to(have_key(u("sessionId"), u("SESSIONID")))
        expect(archive).to(be_an(Archive))
        expect(archive).to(
            have_property(u("id"), u("30b3ebf1-ba36-4f5b-8def-6f70d9986fe9"))
        )
        expect(archive).to(have_property(u("name"), u("")))
        expect(archive).to(have_property(u("status"), u("started")))
        expect(archive).to(have_property(u("session_id"), u("SESSIONID")))
        expect(archive).to(have_property(u("partner_id"), 123456))
        if PY2:
            created_at = datetime.datetime.fromtimestamp(1395183243, pytz.UTC)
        if PY3:
            created_at = datetime.datetime.fromtimestamp(
                1395183243, datetime.timezone.utc
            )
        expect(archive).to(have_property(u("created_at"), created_at))
        expect(archive).to(have_property(u("size"), 0))
        expect(archive).to(have_property(u("has_audio"), True))
        expect(archive).to(have_property(u("has_video"), True))
        expect(archive).to(have_property(u("url"), None))
        expect(archive).to(have_property(u("max_bitrate"), 2000000))

    @httpretty.activate
    def test_start_archive_with_name(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/archive").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "createdAt" : 1395183243556,
                            "duration" : 0,
                            "id" : "30b3ebf1-ba36-4f5b-8def-6f70d9986fe9",
                            "name" : "ARCHIVE NAME",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 0,
                            "status" : "started",
                            "hasAudio": true,
                            "hasVideo": true,
                            "outputMode": "composed",
                            "url" : null
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive = self.opentok.start_archive(self.session_id, name=u("ARCHIVE NAME"))

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        # non-deterministic json encoding. have to decode to test it properly
        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))
        expect(body).to(have_key(u("sessionId"), u("SESSIONID")))
        expect(body).to(have_key(u("name"), u("ARCHIVE NAME")))
        expect(archive).to(be_an(Archive))
        expect(archive).to(
            have_property(u("id"), u("30b3ebf1-ba36-4f5b-8def-6f70d9986fe9"))
        )
        expect(archive).to(have_property(u("name"), ("ARCHIVE NAME")))
        expect(archive).to(have_property(u("status"), u("started")))
        expect(archive).to(have_property(u("session_id"), u("SESSIONID")))
        expect(archive).to(have_property(u("partner_id"), 123456))
        if PY2:
            created_at = datetime.datetime.fromtimestamp(1395183243, pytz.UTC)
        if PY3:
            created_at = datetime.datetime.fromtimestamp(
                1395183243, datetime.timezone.utc
            )
        expect(archive).to(have_property(u("created_at"), equal(created_at)))
        expect(archive).to(have_property(u("size"), equal(0)))
        expect(archive).to(have_property(u("duration"), equal(0)))
        expect(archive).to(have_property(u("url"), equal(None)))

    @httpretty.activate
    def test_start_archive_with_640x480_resolution(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/archive").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "createdAt" : 1395183243556,
                            "duration" : 0,
                            "id" : "30b3ebf1-ba36-4f5b-8def-6f70d9986fe9",
                            "name" : "ARCHIVE NAME",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 0,
                            "status" : "started",
                            "hasAudio": true,
                            "hasVideo": true,
                            "outputMode": "composed",
                            "url" : null,
                            "resolution": "640x480"
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive = self.opentok.start_archive(self.session_id, resolution="640x480")

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        # non-deterministic json encoding. have to decode to test it properly
        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))
        expect(body).to(have_key(u("sessionId"), u("SESSIONID")))
        expect(body).to(have_key(u("resolution"), u("640x480")))
        expect(archive).to(be_an(Archive))
        expect(archive).to(
            have_property(u("id"), u("30b3ebf1-ba36-4f5b-8def-6f70d9986fe9"))
        )
        expect(archive).to(have_property(u("resolution"), "640x480"))
        expect(archive).to(have_property(u("status"), u("started")))
        expect(archive).to(have_property(u("session_id"), u("SESSIONID")))
        expect(archive).to(have_property(u("partner_id"), 123456))
        if PY2:
            created_at = datetime.datetime.fromtimestamp(1395183243, pytz.UTC)
        if PY3:
            created_at = datetime.datetime.fromtimestamp(
                1395183243, datetime.timezone.utc
            )
        expect(archive).to(have_property(u("created_at"), equal(created_at)))
        expect(archive).to(have_property(u("size"), equal(0)))
        expect(archive).to(have_property(u("duration"), equal(0)))
        expect(archive).to(have_property(u("url"), equal(None)))

    @httpretty.activate
    def test_start_archive_with_1280x720_resolution(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/archive").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "createdAt" : 1395183243556,
                            "duration" : 0,
                            "id" : "30b3ebf1-ba36-4f5b-8def-6f70d9986fe9",
                            "name" : "ARCHIVE NAME",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 0,
                            "status" : "started",
                            "hasAudio": true,
                            "hasVideo": true,
                            "outputMode": "composed",
                            "url" : null,
                            "resolution": "1280x720"
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive = self.opentok.start_archive(self.session_id, resolution="1280x720")

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        # non-deterministic json encoding. have to decode to test it properly
        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))
        expect(body).to(have_key(u("sessionId"), u("SESSIONID")))
        expect(body).to(have_key(u("resolution"), u("1280x720")))
        expect(archive).to(be_an(Archive))
        expect(archive).to(
            have_property(u("id"), u("30b3ebf1-ba36-4f5b-8def-6f70d9986fe9"))
        )
        expect(archive).to(have_property(u("resolution"), "1280x720"))
        expect(archive).to(have_property(u("status"), u("started")))
        expect(archive).to(have_property(u("session_id"), u("SESSIONID")))
        expect(archive).to(have_property(u("partner_id"), 123456))
        if PY2:
            created_at = datetime.datetime.fromtimestamp(1395183243, pytz.UTC)
        if PY3:
            created_at = datetime.datetime.fromtimestamp(
                1395183243, datetime.timezone.utc
            )
        expect(archive).to(have_property(u("created_at"), equal(created_at)))
        expect(archive).to(have_property(u("size"), equal(0)))
        expect(archive).to(have_property(u("duration"), equal(0)))
        expect(archive).to(have_property(u("url"), equal(None)))

    def test_start_archive_individual_and_resolution_throws_error(self):
        with pytest.raises(OpenTokException):
            self.opentok.start_archive(
                session_id=self.session_id,
                output_mode=OutputModes.individual,
                resolution="640x480",
            )

        with pytest.raises(OpenTokException):
            self.opentok.start_archive(
                session_id=self.session_id,
                output_mode=OutputModes.individual,
                resolution="1280x720",
            )

    @httpretty.activate
    def test_start_voice_archive(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/archive").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "createdAt" : 1395183243556,
                            "duration" : 0,
                            "id" : "30b3ebf1-ba36-4f5b-8def-6f70d9986fe9",
                            "name" : "ARCHIVE NAME",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 0,
                            "status" : "started",
                            "hasAudio": true,
                            "hasVideo": false,
                            "outputMode": "composed",
                            "url" : null
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive = self.opentok.start_archive(
            self.session_id, name=u("ARCHIVE NAME"), has_video=False
        )

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        # non-deterministic json encoding. have to decode to test it properly
        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))
        expect(body).to(have_key(u("sessionId"), u("SESSIONID")))
        expect(body).to(have_key(u("name"), u("ARCHIVE NAME")))
        expect(archive).to(be_an(Archive))
        expect(archive).to(
            have_property(u("id"), u("30b3ebf1-ba36-4f5b-8def-6f70d9986fe9"))
        )
        expect(archive).to(have_property(u("name"), ("ARCHIVE NAME")))
        expect(archive).to(have_property(u("status"), u("started")))
        expect(archive).to(have_property(u("session_id"), u("SESSIONID")))
        expect(archive).to(have_property(u("partner_id"), 123456))
        if PY2:
            created_at = datetime.datetime.fromtimestamp(1395183243, pytz.UTC)
        if PY3:
            created_at = datetime.datetime.fromtimestamp(
                1395183243, datetime.timezone.utc
            )
        expect(archive).to(have_property(u("created_at"), created_at))
        expect(archive).to(have_property(u("size"), 0))
        expect(archive).to(have_property(u("duration"), 0))
        expect(archive).to(have_property(u("has_audio"), True))
        expect(archive).to(have_property(u("has_video"), False))
        expect(archive).to(have_property(u("url"), None))

    @httpretty.activate
    def test_start_individual_archive(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/archive").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "createdAt" : 1395183243556,
                            "duration" : 0,
                            "id" : "30b3ebf1-ba36-4f5b-8def-6f70d9986fe9",
                            "name" : "ARCHIVE NAME",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 0,
                            "status" : "started",
                            "hasAudio": true,
                            "hasVideo": true,
                            "outputMode": "individual",
                            "url" : null
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive = self.opentok.start_archive(
            self.session_id, name=u("ARCHIVE NAME"), output_mode=OutputModes.individual
        )

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        # non-deterministic json encoding. have to decode to test it properly
        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))
        expect(body).to(have_key(u("sessionId"), u("SESSIONID")))
        expect(body).to(have_key(u("name"), u("ARCHIVE NAME")))
        expect(archive).to(be_an(Archive))
        expect(archive).to(
            have_property(u("id"), u("30b3ebf1-ba36-4f5b-8def-6f70d9986fe9"))
        )
        expect(archive).to(have_property(u("name"), ("ARCHIVE NAME")))
        expect(archive).to(have_property(u("status"), u("started")))
        expect(archive).to(have_property(u("session_id"), u("SESSIONID")))
        expect(archive).to(have_property(u("partner_id"), 123456))
        if PY2:
            created_at = datetime.datetime.fromtimestamp(1395183243, pytz.UTC)
        if PY3:
            created_at = datetime.datetime.fromtimestamp(
                1395183243, datetime.timezone.utc
            )
        expect(archive).to(have_property(u("created_at"), created_at))
        expect(archive).to(have_property(u("size"), 0))
        expect(archive).to(have_property(u("duration"), 0))
        expect(archive).to(have_property(u("has_audio"), True))
        expect(archive).to(have_property(u("has_video"), True))
        expect(archive).to(have_property(u("output_mode"), OutputModes.individual))
        expect(archive).to(have_property(u("url"), None))

    @httpretty.activate
    def test_start_composed_archive(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/archive").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "createdAt" : 1395183243556,
                            "duration" : 0,
                            "id" : "30b3ebf1-ba36-4f5b-8def-6f70d9986fe9",
                            "name" : "ARCHIVE NAME",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 0,
                            "status" : "started",
                            "hasAudio": true,
                            "hasVideo": true,
                            "outputMode": "composed",
                            "url" : null,
                            "maxBitrate": 2000000
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive = self.opentok.start_archive(
            self.session_id, name=u("ARCHIVE NAME"), output_mode=OutputModes.composed
        )

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        # non-deterministic json encoding. have to decode to test it properly
        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))
        expect(body).to(have_key(u("sessionId"), u("SESSIONID")))
        expect(body).to(have_key(u("name"), u("ARCHIVE NAME")))
        expect(archive).to(be_an(Archive))
        expect(archive).to(
            have_property(u("id"), u("30b3ebf1-ba36-4f5b-8def-6f70d9986fe9"))
        )
        expect(archive).to(have_property(u("name"), ("ARCHIVE NAME")))
        expect(archive).to(have_property(u("status"), u("started")))
        expect(archive).to(have_property(u("session_id"), u("SESSIONID")))
        expect(archive).to(have_property(u("partner_id"), 123456))
        if PY2:
            created_at = datetime.datetime.fromtimestamp(1395183243, pytz.UTC)
        if PY3:
            created_at = datetime.datetime.fromtimestamp(
                1395183243, datetime.timezone.utc
            )
        expect(archive).to(have_property(u("created_at"), created_at))
        expect(archive).to(have_property(u("size"), 0))
        expect(archive).to(have_property(u("duration"), 0))
        expect(archive).to(have_property(u("has_audio"), True))
        expect(archive).to(have_property(u("has_video"), True))
        expect(archive).to(have_property(u("output_mode"), OutputModes.composed))
        expect(archive).to(have_property(u("url"), None))
        expect(archive).to(have_property(u("max_bitrate"), 2000000))

    @httpretty.activate
    def test_start_archive_with_layout(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/archive").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "createdAt" : 1395183243556,
                            "duration" : 0,
                            "id" : "30b3ebf1-ba36-4f5b-8def-6f70d9986fe9",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 0,
                            "status" : "started",
                            "hasAudio": true,
                            "hasVideo": true,
                            "outputMode": "composed",
                            "url" : null                            
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive = self.opentok.start_archive(
            self.session_id, layout={"type": "pip", "screenshareType": "horizontal"}
        )

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        # non-deterministic json encoding. have to decode to test it properly
        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))
        expect(body).to(have_key(u("sessionId"), u("SESSIONID")))
        expect(body).to(have_key(u("name"), None))
        expect(body).to(
            have_key(u("layout"), {"type": "pip", "screenshareType": "horizontal"})
        )
        expect(archive).to(be_an(Archive))
        expect(archive).to(
            have_property(u("id"), u("30b3ebf1-ba36-4f5b-8def-6f70d9986fe9"))
        )
        expect(archive).to(have_property(u("name"), ("")))
        expect(archive).to(have_property(u("status"), u("started")))
        expect(archive).to(have_property(u("session_id"), u("SESSIONID")))
        expect(archive).to(have_property(u("partner_id"), 123456))
        if PY2:
            created_at = datetime.datetime.fromtimestamp(1395183243, pytz.UTC)
        if PY3:
            created_at = datetime.datetime.fromtimestamp(
                1395183243, datetime.timezone.utc
            )
        expect(archive).to(have_property(u("created_at"), equal(created_at)))
        expect(archive).to(have_property(u("size"), equal(0)))
        expect(archive).to(have_property(u("duration"), equal(0)))
        expect(archive).to(have_property(u("url"), equal(None)))

    @httpretty.activate
    def test_stop_archive(self):
        archive_id = u("30b3ebf1-ba36-4f5b-8def-6f70d9986fe9")
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/archive/{1}/stop").format(
                self.api_key, archive_id
            ),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "createdAt" : 1395183243000,
                            "duration" : 0,
                            "id" : "30b3ebf1-ba36-4f5b-8def-6f70d9986fe9",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 0,
                            "status" : "stopped",
                            "hasAudio": true,
                            "hasVideo": true,
                            "outputMode": "composed",
                            "url" : null
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive = self.opentok.stop_archive(archive_id)

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        expect(archive).to(be_an(Archive))
        expect(archive).to(have_property(u("id"), archive_id))
        expect(archive).to(have_property(u("name"), u("")))
        expect(archive).to(have_property(u("status"), u("stopped")))
        expect(archive).to(have_property(u("session_id"), u("SESSIONID")))
        expect(archive).to(have_property(u("partner_id"), 123456))
        if PY2:
            created_at = datetime.datetime.fromtimestamp(1395183243, pytz.UTC)
        if PY3:
            created_at = datetime.datetime.fromtimestamp(
                1395183243, datetime.timezone.utc
            )
        expect(archive).to(have_property(u("created_at"), created_at))
        expect(archive).to(have_property(u("size"), 0))
        expect(archive).to(have_property(u("duration"), 0))
        expect(archive).to(have_property(u("url"), None))

    @httpretty.activate
    def test_delete_archive(self):
        archive_id = u("30b3ebf1-ba36-4f5b-8def-6f70d9986fe9")
        httpretty.register_uri(
            httpretty.DELETE,
            u("https://api.opentok.com/v2/project/{0}/archive/{1}").format(
                self.api_key, archive_id
            ),
            body=u(""),
            status=204,
        )

        self.opentok.delete_archive(archive_id)

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )

    @httpretty.activate
    def test_find_archive(self):
        archive_id = u("f6e7ee58-d6cf-4a59-896b-6d56b158ec71")
        httpretty.register_uri(
            httpretty.GET,
            u("https://api.opentok.com/v2/project/{0}/archive/{1}").format(
                self.api_key, archive_id
            ),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "createdAt" : 1395187836000,
                            "duration" : 62,
                            "id" : "f6e7ee58-d6cf-4a59-896b-6d56b158ec71",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 8347554,
                            "status" : "available",
                            "hasAudio": true,
                            "hasVideo": true,
                            "outputMode": "composed",
                            "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2Ff6e7ee58-d6cf-4a59-896b-6d56b158ec71%2Farchive.mp4?Expires=1395194362&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive = self.opentok.get_archive(archive_id)

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        expect(archive).to(be_an(Archive))
        expect(archive).to(have_property(u("id"), archive_id))
        expect(archive).to(have_property(u("name"), u("")))
        expect(archive).to(have_property(u("status"), u("available")))
        expect(archive).to(have_property(u("session_id"), u("SESSIONID")))
        expect(archive).to(have_property(u("partner_id"), 123456))
        if PY2:
            created_at = datetime.datetime.fromtimestamp(1395187836, pytz.UTC)
        if PY3:
            created_at = datetime.datetime.fromtimestamp(
                1395187836, datetime.timezone.utc
            )
        expect(archive).to(have_property(u("created_at"), created_at))
        expect(archive).to(have_property(u("size"), 8347554))
        expect(archive).to(have_property(u("duration"), 62))
        expect(archive).to(
            have_property(
                u("url"),
                u(
                    "http://tokbox.com.archive2.s3.amazonaws.com/123456%2Ff6e7ee58-d6cf-4a59-896b-6d56b158ec71%2Farchive.mp4?Expires=1395194362&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                ),
            )
        )

    @httpretty.activate
    def test_find_archives(self):
        httpretty.register_uri(
            httpretty.GET,
            u("https://api.opentok.com/v2/project/{0}/archive").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "count" : 6,
                            "items" : [ {
                            "createdAt" : 1395187930000,
                            "duration" : 22,
                            "id" : "ef546c5a-4fd7-4e59-ab3d-f1cfb4148d1d",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 2909274,
                            "status" : "available",
                            "hasAudio": true,
                            "hasVideo": true,
                            "outputMode": "composed",
                            "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2Fef546c5a-4fd7-4e59-ab3d-f1cfb4148d1d%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }, {
                            "createdAt" : 1395187910000,
                            "duration" : 14,
                            "id" : "5350f06f-0166-402e-bc27-09ba54948512",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 1952651,
                            "status" : "available",
                            "hasAudio": true,
                            "hasVideo": true,
                            "outputMode": "composed",
                            "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2F5350f06f-0166-402e-bc27-09ba54948512%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }, {
                            "createdAt" : 1395187836000,
                            "duration" : 62,
                            "id" : "f6e7ee58-d6cf-4a59-896b-6d56b158ec71",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 8347554,
                            "status" : "available",
                            "hasAudio": true,
                            "hasVideo": true,
                            "outputMode": "composed",
                            "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2Ff6e7ee58-d6cf-4a59-896b-6d56b158ec71%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }, {
                            "createdAt" : 1395183243000,
                            "duration" : 544,
                            "id" : "30b3ebf1-ba36-4f5b-8def-6f70d9986fe9",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 78499758,
                            "status" : "available",
                            "hasAudio": true,
                            "hasVideo": true,
                            "outputMode": "composed",
                            "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2F30b3ebf1-ba36-4f5b-8def-6f70d9986fe9%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }, {
                            "createdAt" : 1394396753000,
                            "duration" : 24,
                            "id" : "b8f64de1-e218-4091-9544-4cbf369fc238",
                            "name" : "showtime again",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 2227849,
                            "status" : "available",
                            "hasAudio": true,
                            "hasVideo": true,
                            "outputMode": "composed",
                            "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2Fb8f64de1-e218-4091-9544-4cbf369fc238%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }, {
                            "createdAt" : 1394321113000,
                            "duration" : 1294,
                            "id" : "832641bf-5dbf-41a1-ad94-fea213e59a92",
                            "name" : "showtime",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 42165242,
                            "status" : "available",
                            "hasAudio": true,
                            "hasVideo": true,
                            "outputMode": "composed",
                            "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2F832641bf-5dbf-41a1-ad94-fea213e59a92%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            } ]
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive_list = self.opentok.get_archives()

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        expect(archive_list).to(be_an(ArchiveList))
        expect(archive_list).to(have_property(u("count"), 6))
        expect(list(archive_list.items)).to(have_length(6))
        # TODO: we could inspect each item in the list

    @httpretty.activate
    def test_find_archives_with_offset(self):
        httpretty.register_uri(
            httpretty.GET,
            u("https://api.opentok.com/v2/project/{0}/archive").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "count" : 6,
                            "items" : [ {
                            "createdAt" : 1395183243000,
                            "duration" : 544,
                            "id" : "30b3ebf1-ba36-4f5b-8def-6f70d9986fe9",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 78499758,
                            "status" : "available",
                            "hasAudio": true,
                            "hasVideo": true,
                            "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2F30b3ebf1-ba36-4f5b-8def-6f70d9986fe9%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }, {
                            "createdAt" : 1394396753000,
                            "duration" : 24,
                            "id" : "b8f64de1-e218-4091-9544-4cbf369fc238",
                            "name" : "showtime again",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 2227849,
                            "status" : "available",
                            "hasAudio": true,
                            "hasVideo": true,
                            "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2Fb8f64de1-e218-4091-9544-4cbf369fc238%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }, {
                            "createdAt" : 1394321113000,
                            "duration" : 1294,
                            "id" : "832641bf-5dbf-41a1-ad94-fea213e59a92",
                            "name" : "showtime",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 42165242,
                            "status" : "available",
                            "hasAudio": true,
                            "hasVideo": true,
                            "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2F832641bf-5dbf-41a1-ad94-fea213e59a92%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            } ]
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive_list = self.opentok.get_archives(offset=3)

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        expect(httpretty.last_request()).to(
            have_property(u("querystring"), {u("offset"): [u("3")]})
        )
        expect(archive_list).to(be_an(ArchiveList))
        expect(archive_list).to(have_property(u("count"), 6))
        expect(list(archive_list.items)).to(have_length(3))
        # TODO: we could inspect each item in the list

    @httpretty.activate
    def test_find_archives_with_count(self):
        httpretty.register_uri(
            httpretty.GET,
            u("https://api.opentok.com/v2/project/{0}/archive").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "count" : 6,
                            "items" : [ {
                            "createdAt" : 1395187930000,
                            "duration" : 22,
                            "id" : "ef546c5a-4fd7-4e59-ab3d-f1cfb4148d1d",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 2909274,
                            "status" : "available",
                            "hasAudio": true,
                            "hasVideo": true,
                            "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2Fef546c5a-4fd7-4e59-ab3d-f1cfb4148d1d%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }, {
                            "createdAt" : 1395187910000,
                            "duration" : 14,
                            "id" : "5350f06f-0166-402e-bc27-09ba54948512",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 1952651,
                            "status" : "available",
                            "hasAudio": true,
                            "hasVideo": true,
                            "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2F5350f06f-0166-402e-bc27-09ba54948512%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            } ]
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive_list = self.opentok.get_archives(count=2)

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        expect(httpretty.last_request()).to(
            have_property(u("querystring"), {u("count"): [u("2")]})
        )
        expect(archive_list).to(be_an(ArchiveList))
        expect(archive_list).to(have_property(u("count"), 6))
        expect(list(archive_list.items)).to(have_length(2))
        # TODO: we could inspect each item in the list

    @httpretty.activate
    def test_find_archives_with_offset_and_count(self):
        httpretty.register_uri(
            httpretty.GET,
            u("https://api.opentok.com/v2/project/{0}/archive").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "count" : 6,
                            "items" : [ {
                            "createdAt" : 1395187836000,
                            "duration" : 62,
                            "id" : "f6e7ee58-d6cf-4a59-896b-6d56b158ec71",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 8347554,
                            "status" : "available",
                            "hasAudio": true,
                            "hasVideo": true,
                            "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2Ff6e7ee58-d6cf-4a59-896b-6d56b158ec71%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }, {
                            "createdAt" : 1395183243000,
                            "duration" : 544,
                            "id" : "30b3ebf1-ba36-4f5b-8def-6f70d9986fe9",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 78499758,
                            "status" : "available",
                            "hasAudio": true,
                            "hasVideo": true,
                            "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2F30b3ebf1-ba36-4f5b-8def-6f70d9986fe9%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }, {
                            "createdAt" : 1394396753000,
                            "duration" : 24,
                            "id" : "b8f64de1-e218-4091-9544-4cbf369fc238",
                            "name" : "showtime again",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 2227849,
                            "status" : "available",
                            "hasAudio": true,
                            "hasVideo": true,
                            "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2Fb8f64de1-e218-4091-9544-4cbf369fc238%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }, {
                            "createdAt" : 1394321113000,
                            "duration" : 1294,
                            "id" : "832641bf-5dbf-41a1-ad94-fea213e59a92",
                            "name" : "showtime",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 42165242,
                            "status" : "available",
                            "hasAudio": true,
                            "hasVideo": true,
                            "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2F832641bf-5dbf-41a1-ad94-fea213e59a92%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            } ]
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive_list = self.opentok.get_archives(count=4, offset=2)

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        expect(httpretty.last_request()).to(
            have_property(u("querystring"), {u("offset"): [u("2")], u("count"): [u("4")]})
        )
        expect(archive_list).to(be_an(ArchiveList))
        expect(archive_list).to(have_property(u("count"), 6))
        expect(list(archive_list.items)).to(have_length(4))
        # TODO: we could inspect each item in the list

    @httpretty.activate
    def test_find_archives_with_sessionid(self):
        """Test get_archives method using session_id parameter"""
        httpretty.register_uri(
            httpretty.GET,
            u("https://api.opentok.com/v2/project/{0}/archive").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "count" : 4,
                            "items" : [ {
                                "createdAt" : 1395187836000,
                                "duration" : 62,
                                "id" : "f6e7ee58-d6cf-4a59-896b-6d56b158ec71",
                                "name" : "",
                                "partnerId" : 123456,
                                "reason" : "",
                                "sessionId" : "SESSIONID",
                                "size" : 8347554,
                                "status" : "available",
                                "hasAudio": true,
                                "hasVideo": true,
                                "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2Ff6e7ee58-d6cf-4a59-896b-6d56b158ec71%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }, {
                                "createdAt" : 1395183243000,
                                "duration" : 544,
                                "id" : "30b3ebf1-ba36-4f5b-8def-6f70d9986fe9",
                                "name" : "",
                                "partnerId" : 123456,
                                "reason" : "",
                                "sessionId" : "SESSIONID",
                                "size" : 78499758,
                                "status" : "available",
                                "hasAudio": true,
                                "hasVideo": true,
                                "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2F30b3ebf1-ba36-4f5b-8def-6f70d9986fe9%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }, {
                                "createdAt" : 1394396753000,
                                "duration" : 24,
                                "id" : "b8f64de1-e218-4091-9544-4cbf369fc238",
                                "name" : "showtime again",
                                "partnerId" : 123456,
                                "reason" : "",
                                "sessionId" : "SESSIONID",
                                "size" : 2227849,
                                "status" : "available",
                                "hasAudio": true,
                                "hasVideo": true,
                                "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2Fb8f64de1-e218-4091-9544-4cbf369fc238%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }, {
                                "createdAt" : 1394321113000,
                                "duration" : 1294,
                                "id" : "832641bf-5dbf-41a1-ad94-fea213e59a92",
                                "name" : "showtime",
                                "partnerId" : 123456,
                                "reason" : "",
                                "sessionId" : "SESSIONID",
                                "size" : 42165242,
                                "status" : "available",
                                "hasAudio": true,
                                "hasVideo": true,
                                "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2F832641bf-5dbf-41a1-ad94-fea213e59a92%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }]
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive_list = self.opentok.get_archives(session_id="SESSIONID")

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        expect(httpretty.last_request()).to(
            have_property(u("querystring"), {u("sessionId"): [u("SESSIONID")]})
        )
        expect(archive_list).to(be_an(ArchiveList))
        expect(archive_list).to(have_property(u("count"), 4))
        expect(list(archive_list.items)).to(have_length(4))

    @httpretty.activate
    def test_find_archives_with_offset_count_sessionId(self):
        """Test get_archives method using all parameters: offset, count and sessionId"""
        httpretty.register_uri(
            httpretty.GET,
            u("https://api.opentok.com/v2/project/{0}/archive").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "count" : 2,
                            "items" : [ {
                                "createdAt" : 1394396753000,
                                "duration" : 24,
                                "id" : "b8f64de1-e218-4091-9544-4cbf369fc238",
                                "name" : "showtime again",
                                "partnerId" : 123456,
                                "reason" : "",
                                "sessionId" : "SESSIONID",
                                "size" : 2227849,
                                "status" : "available",
                                "hasAudio": true,
                                "hasVideo": true,
                                "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2Fb8f64de1-e218-4091-9544-4cbf369fc238%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }, {
                                "createdAt" : 1394321113000,
                                "duration" : 1294,
                                "id" : "832641bf-5dbf-41a1-ad94-fea213e59a92",
                                "name" : "showtime",
                                "partnerId" : 123456,
                                "reason" : "",
                                "sessionId" : "SESSIONID",
                                "size" : 42165242,
                                "status" : "available",
                                "hasAudio": true,
                                "hasVideo": true,
                                "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2F832641bf-5dbf-41a1-ad94-fea213e59a92%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }]
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive_list = self.opentok.get_archives(
            offset=2, count=2, session_id="SESSIONID"
        )

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        expect(httpretty.last_request()).to(
            have_property(
                u("querystring"),
                {
                    u("offset"): [u("2")],
                    u("count"): [u("2")],
                    u("sessionId"): [u("SESSIONID")],
                },
            )
        )
        expect(archive_list).to(be_an(ArchiveList))
        expect(archive_list).to(have_property(u("count"), 2))
        expect(list(archive_list.items)).to(have_length(2))

    @httpretty.activate
    def test_find_archives_alternative_method(self):
        """Test list_archives method using all parameters: offset, count and sessionId"""
        httpretty.register_uri(
            httpretty.GET,
            u("https://api.opentok.com/v2/project/{0}/archive").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "count" : 2,
                            "items" : [ {
                                "createdAt" : 1394396753000,
                                "duration" : 24,
                                "id" : "b8f64de1-e218-4091-9544-4cbf369fc238",
                                "name" : "showtime again",
                                "partnerId" : 123456,
                                "reason" : "",
                                "sessionId" : "SESSIONID",
                                "size" : 2227849,
                                "status" : "available",
                                "hasAudio": true,
                                "hasVideo": true,
                                "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2Fb8f64de1-e218-4091-9544-4cbf369fc238%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }, {
                                "createdAt" : 1394321113000,
                                "duration" : 1294,
                                "id" : "832641bf-5dbf-41a1-ad94-fea213e59a92",
                                "name" : "showtime",
                                "partnerId" : 123456,
                                "reason" : "",
                                "sessionId" : "SESSIONID",
                                "size" : 42165242,
                                "status" : "available",
                                "hasAudio": true,
                                "hasVideo": true,
                                "url" : "http://tokbox.com.archive2.s3.amazonaws.com/123456%2F832641bf-5dbf-41a1-ad94-fea213e59a92%2Farchive.mp4?Expires=1395188695&AWSAccessKeyId=AKIAI6LQCPIXYVWCQV6Q&Signature=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
                            }]
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive_list = self.opentok.list_archives(
            offset=2, count=2, session_id="SESSIONID"
        )

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )
        expect(httpretty.last_request()).to(
            have_property(
                u("querystring"),
                {
                    u("offset"): [u("2")],
                    u("count"): [u("2")],
                    u("sessionId"): [u("SESSIONID")],
                },
            )
        )
        expect(archive_list).to(be_an(ArchiveList))
        expect(archive_list).to(have_property(u("count"), 2))
        expect(list(archive_list.items)).to(have_length(2))

    @httpretty.activate
    def test_find_paused_archive(self):
        archive_id = u("f6e7ee58-d6cf-4a59-896b-6d56b158ec71")
        httpretty.register_uri(
            httpretty.GET,
            u("https://api.opentok.com/v2/project/{0}/archive/{1}").format(
                self.api_key, archive_id
            ),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "createdAt" : 1395187836000,
                            "duration" : 62,
                            "id" : "f6e7ee58-d6cf-4a59-896b-6d56b158ec71",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 8347554,
                            "status" : "paused",
                            "hasAudio": true,
                            "hasVideo": true,
                            "url" : null
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive = self.opentok.get_archive(archive_id)

        expect(archive).to(be_an(Archive))
        expect(archive).to(have_property(u("status"), u("paused")))

    @httpretty.activate
    def test_find_expired_archive(self):
        archive_id = u("f6e7ee58-d6cf-4a59-896b-6d56b158ec71")
        httpretty.register_uri(
            httpretty.GET,
            u("https://api.opentok.com/v2/project/{0}/archive/{1}").format(
                self.api_key, archive_id
            ),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "createdAt" : 1395187836000,
                            "duration" : 62,
                            "id" : "f6e7ee58-d6cf-4a59-896b-6d56b158ec71",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 8347554,
                            "status" : "expired",
                            "hasAudio": true,
                            "hasVideo": true,
                            "url" : null
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive = self.opentok.get_archive(archive_id)

        expect(archive).to(be_an(Archive))
        expect(archive).to(have_property(u("status"), u("expired")))

    @httpretty.activate
    def test_find_archive_with_unknown_properties(self):
        archive_id = u("f6e7ee58-d6cf-4a59-896b-6d56b158ec71")
        httpretty.register_uri(
            httpretty.GET,
            u("https://api.opentok.com/v2/project/{0}/archive/{1}").format(
                self.api_key, archive_id
            ),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "createdAt" : 1395187836000,
                            "duration" : 62,
                            "id" : "f6e7ee58-d6cf-4a59-896b-6d56b158ec71",
                            "name" : "",
                            "partnerId" : 123456,
                            "reason" : "",
                            "sessionId" : "SESSIONID",
                            "size" : 8347554,
                            "status" : "expired",
                            "url" : null,
                            "hasAudio": true,
                            "hasVideo": true,
                            "notarealproperty" : "not a real value"
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        archive = self.opentok.get_archive(archive_id)

        expect(archive).to(be_an(Archive))

    @httpretty.activate
    def test_set_archive_layout(self):
        """Test set archive layout functionality"""
        archive_id = u("f6e7ee58-d6cf-4a59-896b-6d56b158ec71")

        httpretty.register_uri(
            httpretty.PUT,
            u("https://api.opentok.com/v2/project/{0}/archive/{1}/layout").format(
                self.api_key, archive_id
            ),
            status=200,
            content_type=u("application/json"),
        )

        self.opentok.set_archive_layout(archive_id, "horizontalPresentation")

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )

    @httpretty.activate
    def test_set_archive_screenshare_type(self):
        """Test set archive layout functionality"""
        archive_id = u("f6e7ee58-d6cf-4a59-896b-6d56b158ec71")

        httpretty.register_uri(
            httpretty.PUT,
            u("https://api.opentok.com/v2/project/{0}/archive/{1}/layout").format(
                self.api_key, archive_id
            ),
            status=200,
            content_type=u("application/json"),
        )

        self.opentok.set_archive_layout(
            archive_id, "bestFit", screenshare_type="horizontalPresentation"
        )

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )

        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))

        expect(body).to(have_key(u("type"), u("bestFit")))
        expect(body).to_not(have_key(u("stylesheet")))
        expect(body).to(have_key(u("screenshareType"), u("horizontalPresentation")))

    @httpretty.activate
    def test_set_custom_archive_layout(self):
        """Test set a custom archive layout specifying the 'stylesheet' parameter"""
        archive_id = u("f6e7ee58-d6cf-4a59-896b-6d56b158ec71")

        httpretty.register_uri(
            httpretty.PUT,
            u("https://api.opentok.com/v2/project/{0}/archive/{1}/layout").format(
                self.api_key, archive_id
            ),
            status=200,
            content_type=u("application/json"),
        )

        self.opentok.set_archive_layout(
            archive_id,
            "custom",
            "stream.instructor {position: absolute; width: 100%;  height:50%;}",
        )

        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        )

    @httpretty.activate
    def test_start_archive_with_streammode_auto(self):
        url = f"https://api.opentok.com/v2/project/{self.api_key}/archive"

        httpretty.register_uri(
            httpretty.POST,
            url,
            responses=[
                httpretty.Response(
                    body=json.dumps({"streamMode": "auto"}),
                    content_type="application/json",
                    status=200,
                )
            ],
        )

        response = requests.post(url)

        response.status_code.should.equal(200)
        response.json().should.equal({"streamMode": "auto"})
        response.headers["Content-Type"].should.equal("application/json")

    @httpretty.activate
    def test_start_archive_with_streammode_manual(self):
        url = f"https://api.opentok.com/v2/project/{self.api_key}/archive"

        httpretty.register_uri(
            httpretty.POST,
            url,
            responses=[
                httpretty.Response(
                    body=json.dumps({"streamMode": "manual"}),
                    content_type="application/json",
                    status=200,
                )
            ],
        )

        response = requests.post(url)

        response.status_code.should.equal(200)
        response.json().should.equal({"streamMode": "manual"})
        response.headers["Content-Type"].should.equal("application/json")

    @httpretty.activate
    def test_set_archive_layout_throws_exception(self):
        """Test invalid request in set archive layout"""
        archive_id = u("f6e7ee58-d6cf-4a59-896b-6d56b158ec71")

        httpretty.register_uri(
            httpretty.PUT,
            u("https://api.opentok.com/v2/project/{0}/archive/{1}/layout").format(
                self.api_key, archive_id
            ),
            status=400,
            content_type=u("application/json"),
        )

        with pytest.raises(ArchiveError):
            self.opentok.set_archive_layout(archive_id, "horizontalPresentation")



import unittest
from six import text_type, u, b, PY2, PY3
from expects import *

from opentok import Client, __version__
import time
from jwt import decode


class JwtCustomClaimsTest(unittest.TestCase):
    def setUp(self):
        self.api_key = u("123456")
        self.api_secret = u("1234567890abcdef1234567890abcdef1234567890")
        self.session_id = u(
            "1_MX4xMjM0NTZ-flNhdCBNYXIgMTUgMTQ6NDI6MjMgUERUIDIwMTR-MC40OTAxMzAyNX4"
        )
        self.opentok = Client(self.api_key, self.api_secret)

    def test_livetime_custom_claim(self):
        self.opentok.jwt_livetime = 5  # Token will expire 5 minutes in the future
        jwt_token = self.opentok._create_jwt_auth_header()
        claims = decode(jwt_token, self.api_secret, algorithms=[u("HS256")])
        expect(claims).to(have_key(u("exp")))
        expect(int(claims[u("exp")])).to(
            be_above(int(time.time()) + (60 * 4))
        )  # above of 4 min
        expect(int(claims[u("exp")])).to(
            be_below(int(time.time()) + (60 * 6))
        )  # below of 6 min



import unittest

import requests
import httpretty
import random
import string

from opentok import Client


class OpenTokTest(unittest.TestCase):  
    def setUp(self):
        self.api_key = "123456"
        self.api_secret = "1234567890abcdef1234567890abcdef1234567890"
        self.session_id = "SESSIONID"
        self.connection_id = "CONNECTIONID"

        self.opentok = Client(self.api_key, self.api_secret)
        token = string.ascii_letters+string.digits
        self.jwt_token_string = ''.join(random.choice(token[:100]))
        self.stream_id_1 = "Stream1"

   

    @httpretty.activate
    def test_mute_all_exclude_streams(self):
        self.url = "https://api.opentok.com/v2/project/{0}/session/{1}/mute".format(
                                                                                self.api_key, 
                                                                                self.session_id)

        httpretty.register_uri(httpretty.POST, 
                                        self.url, 
                                        responses=[
                                            httpretty.Response(body='{"active": True, "excludedStreams": "excludedStreamIds1"}', 
                                                               content_type="application/json",
                                                               adding_headers= {"x-opentok-auth": self.jwt_token_string},
                                                               status=201)
                                        ])

    
        response = requests.post(self.url)

        response.text.should.equal('{"active": True, "excludedStreams": "excludedStreamIds1"}')
        response.headers["x-opentok-auth"].should.equal(self.jwt_token_string)
        response.headers["Content-Type"].should.equal("application/json")

    @httpretty.activate
    def test_disable_force_mute(self):
        self.url = "https://api.opentok.com/v2/project/{0}/session/{1}/mute".format(
                                                                                self.api_key, 
                                                                                self.session_id)

        httpretty.register_uri(httpretty.POST, 
                                        self.url, 
                                        responses=[
                                            httpretty.Response(body='{}',
                                                                content_type="application/json",
                                                                adding_headers= {"x-opentok-auth": self.jwt_token_string},
                                                                status=201)
                                        ])

    
        response = requests.post(self.url)

        response.text.should.equal('{}')
        response.headers["x-opentok-auth"].should.equal(self.jwt_token_string)
        response.headers["Content-Type"].should.equal("application/json")


    @httpretty.activate
    def test_mute_single_stream(self):
        self.url = "https://api.opentok.com/v2/project/{0}/session/{1}/stream/{2}/mute".format(
                                                                                self.api_key, 
                                                                                self.session_id,
                                                                                self.stream_id_1)

        httpretty.register_uri(httpretty.POST, 
                                        self.url, 
                                        responses=[
                                            httpretty.Response(body='{"session_id": "12345", "stream_id": "abcde"}', 
                                                               content_type="application/json",
                                                               adding_headers= {"x-opentok-auth": self.jwt_token_string},
                                                               status=201)
                                        ])

    
        response = requests.post(self.url)

        response.text.should.equal('{"session_id": "12345", "stream_id": "abcde"}')
        response.headers["x-opentok-auth"].should.equal(self.jwt_token_string)
        response.headers["Content-Type"].should.equal("application/json")


    @httpretty.activate
    def test_mute_stream_response(self):
        self.url = "https://api.opentok.com/v2/project/${0}/session/${1}/stream/${2}/mute".format(
                                                                                                self.api_key, 
                                                                                                self.session_id,
                                                                                                self.stream_id_1)

        httpretty.register_uri(httpretty.POST, 
                                        self.url, 
                                        responses=[
                                            httpretty.Response(body="Testing body of the JSON file", 
                                                               content_type="application/json",
                                                               adding_headers= {"x-opentok-auth": self.jwt_token_string},
                                                               status=201)
                                        ])

    
        response = requests.post(self.url)

        response.status_code.should.equal(201)
        response.text.should.equal("Testing body of the JSON file")
        response.headers["x-opentok-auth"].should.equal(self.jwt_token_string)
        response.headers["Content-Type"].should.equal("application/json")

    @httpretty.activate
    def test_dtmf_all_clients(self):
        self.url = f"https://api.opentok.com/v2/project/{self.api_key}/session/{self.session_id}/play-dtmf"
                                                                            
        httpretty.register_uri(httpretty.POST, 
                                        self.url, 
                                        responses=[
                                            httpretty.Response(body="Testing text matches inside of the JSON file", 
                                                               content_type="application/json",
                                                               adding_headers= {"x-opentok-auth": self.jwt_token_string},
                                                               status=200)
                                        ])

    
        response = requests.post(self.url)

        response.status_code.should.equal(200)
        response.text.should.equal("Testing text matches inside of the JSON file")
        response.headers["x-opentok-auth"].should.equal(self.jwt_token_string)
        response.headers["Content-Type"].should.equal("application/json")

    @httpretty.activate
    def test_dtmf_specific_client(self):
        self.url = f"https://api.opentok.com/v2/project/{self.api_key}/session/{self.session_id}/connection/{self.connection_id}/play-dtmf"
                                                                            
        httpretty.register_uri(httpretty.POST, 
                                        self.url, 
                                        responses=[
                                            httpretty.Response(body="Testing text matches inside of the JSON file", 
                                                               content_type="application/json",
                                                               adding_headers= {"x-opentok-auth": self.jwt_token_string},
                                                               status=200)
                                        ])

    
        response = requests.post(self.url)

        response.status_code.should.equal(200)
        response.text.should.equal("Testing text matches inside of the JSON file")
        response.headers["x-opentok-auth"].should.equal(self.jwt_token_string)
        response.headers["Content-Type"].should.equal("application/json")




import pytest
import unittest
import platform

from opentok import Client


class OpenTokInitializationTest(unittest.TestCase):
    def setUp(self):
        self.api_key = "123456"
        self.api_secret = "1234567890abcdef1234567890abcdef1234567890"
        self.api_url = "http://environment.example.com"

    def test_intialization(self):
        opentok = Client(self.api_key, self.api_secret)
        assert isinstance(opentok, Client)
        self.assertEqual(opentok.proxies, None)

    def test_set_proxies(self):
        opentok = Client(self.api_key, self.api_secret)
        opentok.proxies = {"https": "https://foo.bar"}
        self.assertEqual(opentok.proxies, {"https": "https://foo.bar"})

    def test_initialization_without_required_params(self):
        with pytest.raises(TypeError):
            opentok = Client()

    def test_initialization_with_api_url(self):
        opentok = Client(self.api_key, self.api_secret, self.api_url)
        assert isinstance(opentok, Client)

    def test_initialization_with_numeric_api_key(self):
        opentok = Client(123456, self.api_secret)
        assert isinstance(opentok, Client)

    def test_initialization_with_timeout(self):
        opentok = Client(self.api_key, self.api_secret, timeout=5)
        assert isinstance(opentok, Client)

    def test_user_agent(self):
        opentok = Client(self.api_key, self.api_secret, self.api_url)
        assert (
            opentok.user_agent
            == f"OpenTok-Python-SDK/{opentok.app_version} python/{platform.python_version()}"
        )

    def test_append_to_user_agent(self):
        opentok = Client(self.api_key, self.api_secret, self.api_url)
        opentok.append_to_user_agent("/my_appended_user_agent_string")
        assert (
            opentok.user_agent
            == f"OpenTok-Python-SDK/{opentok.app_version} python/{platform.python_version()}/my_appended_user_agent_string"
        )


if __name__ == "__main__":
    unittest.main()



import unittest
import textwrap
import httpretty
import json
import requests
from sure import expect

from six import u, PY2, PY3
from expects import *
from opentok import Client, Render, RenderList, __version__
from .validate_jwt import validate_jwt_header


class OpenTokRenderTest(unittest.TestCase):
    def setUp(self):
        self.api_key = u("123456")
        self.api_secret = u("1234567890abcdef1234567890abcdef1234567890")
        self.opentok = Client(self.api_key, self.api_secret)
        self.session_id = u("2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4")
        self.token = u("1234-5678-9012")
        self.render_id = u("80abaf0d-25a3-4efc-968f-6268d620668d")

    @httpretty.activate
    def test_start_render(self):
        httpretty.register_uri(
            httpretty.POST,
            u("https://api.opentok.com/v2/project/{0}/render").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "id": "1248e7070b81464c9789f46ad10e7764",
                            "sessionId": "2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4",
                            "projectId": "e2343f23456g34709d2443a234",
                            "createdAt": 1437676551000,
                            "updatedAt": 1437676551000,
                            "url": "https://webapp.customer.com",
                            "resolution": "1280x720",
                            "status": "started",
                            "streamId": "e32445b743678c98230f238" 
                        }
                    """
                )
            ),
            status=202,
            content_type=u("application/json"),
        )

        url = "https://webapp.customer.com"

        render = self.opentok.start_render(self.session_id, self.token, url)
        
        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        
        )
        # non-deterministic json encoding. have to decode to test it properly
        if PY2:
            body = json.loads(httpretty.last_request().body)
        if PY3:
            body = json.loads(httpretty.last_request().body.decode("utf-8"))

        expect(body).to(have_key(u("token")))
        expect(render).to(be_a(Render))
        expect(render).to(
            have_property(u("id"), u("1248e7070b81464c9789f46ad10e7764"))
        )
        expect(render).to(
            have_property(u("sessionId"), u("2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4"))
        )
        expect(render).to(have_property(u("projectId"), u("e2343f23456g34709d2443a234")))
        expect(render).to(have_property(u("createdAt"), 1437676551000))
        expect(render).to(have_property(u("updatedAt"), 1437676551000))
        expect(render).to(have_property(u("resolution"), u("1280x720")))
        expect(render).to(have_property(u("status"), u("started")))
        expect(render).to(have_property(u("streamId"), u("e32445b743678c98230f238")))
    

    @httpretty.activate
    def test_get_render_status(self):
        httpretty.register_uri(
            httpretty.GET,
            u("https://api.opentok.com/v2/project/{0}/render/{1}").format(self.api_key, self.render_id),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "id": "80abaf0d-25a3-4efc-968f-6268d620668d",
                            "sessionId": "2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4",
                            "projectId": "e2343f23456g34709d2443a234",
                            "createdAt": 1437676551000,
                            "updatedAt": 1437676551000,
                            "url": "https://webapp.customer.com",
                            "resolution": "1280x720",
                            "status": "failed",
                            "reason":"Could not load URL"
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        render = self.opentok.get_render(self.render_id)
        
        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )
        expect(httpretty.last_request().headers[u("content-type")]).to(
            equal(u("application/json"))
        
        )

        expect(render).to(be_a(Render))
        expect(render).to(
            have_property(u("id"), u("80abaf0d-25a3-4efc-968f-6268d620668d"))
        )
        expect(render).to(
            have_property(u("sessionId"), u("2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4"))
        )
        expect(render).to(have_property(u("projectId"), u("e2343f23456g34709d2443a234")))
        expect(render).to(have_property(u("createdAt"), 1437676551000))
        expect(render).to(have_property(u("updatedAt"), 1437676551000))
        expect(render).to(have_property(u("resolution"), u("1280x720")))
        expect(render).to(have_property(u("status"), u("failed")))
        expect(render).to(have_property(u("reason"), u("Could not load URL")))

    @httpretty.activate
    def test_stop_render(self):
        httpretty.register_uri(
            httpretty.DELETE,
            u("https://api.opentok.com/v2/project/{0}/render/{1}").format(self.api_key, self.render_id),
            body="",
            status=200,
        )

        response = self.opentok.stop_render(self.render_id)
        
        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )

        assert isinstance(response, requests.Response)
        assert response.status_code == 200

    @httpretty.activate
    def test_list_renders(self):
        httpretty.register_uri(
            httpretty.GET,
            u("https://api.opentok.com/v2/project/{0}/render").format(self.api_key),
            body=textwrap.dedent(
                u(
                    """\
                        {
                            "count":2,
                            "items":[
                                {
                                    "id":"80abaf0d-25a3-4efc-968f-6268d620668d",
                                    "sessionId":"1_MX4yNzA4NjYxMn5-MTU0NzA4MDUyMTEzNn5sOXU5ZnlWYXplRnZGblV4RUo3dXJpZk1-fg",
                                    "projectId":"27086612",
                                    "createdAt":1547080532099,
                                    "updatedAt":1547080532099,
                                    "url": "https://webapp.customer.com",
                                    "resolution": "1280x720",
                                    "status": "started",
                                    "streamId": "d2334b35690a92f78945"
                                },
                                {
                                    "id":"d95f6496-df6e-4f49-86d6-832e00303602",
                                    "sessionId":"2_MX4yNzA4NjYxMn5-MTU0NzA4MDUwMDc2MH5STWRiSE1jZjVoV3lBQU9nN2JuNElUV3V-fg",
                                    "projectId":"27086612",
                                    "createdAt":1547080511760,
                                    "updatedAt":1547080518965,
                                    "url": "https://webapp2.customer.com",
                                    "resolution": "1280x720",
                                    "status":"stopped",
                                    "streamId": "d2334b35690a92f78945",
                                    "reason":"Maximum duration exceeded"
                                }
                            ]
                        }
                    """
                )
            ),
            status=200,
            content_type=u("application/json"),
        )

        render_list = self.opentok.list_renders()
        validate_jwt_header(self, httpretty.last_request().headers[u("x-opentok-auth")])
        expect(httpretty.last_request().headers[u("user-agent")]).to(
            contain(u("OpenTok-Python-SDK/") + __version__)
        )

        expect(render_list).to(be_a(RenderList))
        expect(render_list).to(have_property(u("count"), 2))
        expect(render_list).to(have_property(u("items")))
        expect(render_list.items[0]).to(have_property(u("sessionId"), u("1_MX4yNzA4NjYxMn5-MTU0NzA4MDUyMTEzNn5sOXU5ZnlWYXplRnZGblV4RUo3dXJpZk1-fg")))
        expect(render_list.items[0]).to(have_property(u("createdAt"), u(1547080532099)))
        expect(render_list.items[0]).to(have_property(u("url"), u("https://webapp.customer.com")))
        expect(render_list.items[0]).to(have_property(u("status"), u("started")))
        expect(render_list.items[0]).to(have_property(u("streamId"), u("d2334b35690a92f78945")))
        expect(render_list.items[1]).to(have_property(u("sessionId"), u("2_MX4yNzA4NjYxMn5-MTU0NzA4MDUwMDc2MH5STWRiSE1jZjVoV3lBQU9nN2JuNElUV3V-fg")))
        expect(render_list.items[1]).to(have_property(u("createdAt"), u(1547080511760)))
        expect(render_list.items[1]).to(have_property(u("updatedAt"), u(1547080518965)))
        expect(render_list.items[1]).to(have_property(u("url"), u("https://webapp2.customer.com")))
        expect(render_list.items[1]).to(have_property(u("status"), u("stopped")))
        expect(render_list.items[1]).to(have_property(u("streamId"), u("d2334b35690a92f78945")))
        expect(render_list.items[1]).to(have_property(u("reason"), u("Maximum duration exceeded")))



import unittest
import textwrap
import httpretty
import json
from sure import expect

from expects import *
from opentok import Client, __version__
from opentok.captions import Captions


class OpenTokCaptionsTest(unittest.TestCase):
    def setUp(self):
        self.api_key = "123456"
        self.api_secret = "1234567890abcdef1234567890abcdef1234567890"
        self.opentok = Client(self.api_key, self.api_secret)
        self.session_id = "2_MX4xMDBfjE0Mzc2NzY1NDgwMTJ-TjMzfn4"
        self.token = "1234-5678-9012"
        self.response_body = textwrap.dedent(
            """ \
                {
                    "captionsId": "7c0680fc-6274-4de5-a66f-d0648e8d3ac2"
                }
            """
        )

    @httpretty.activate
    def test_start_captions(self):
        httpretty.register_uri(
            httpretty.POST,
            f"https://api.opentok.com/v2/project/{self.api_key}/captions",
            body=self.response_body,
            status=200,
            content_type="application/json",
        )

        captions = self.opentok.start_captions(
            self.session_id,
            self.token,
            language_code='en-GB',
            max_duration=10000,
            partial_captions=False,
            status_callback_url='https://example.com',
        )

        body = json.loads(httpretty.last_request().body.decode("utf-8"))
        expect(body).to(have_key("token"))

        expect(captions).to(be_a(Captions))
        expect(captions).to(
            have_property("captions_id", "7c0680fc-6274-4de5-a66f-d0648e8d3ac2")
        )

    @httpretty.activate
    def test_stop_captions(self):
        httpretty.register_uri(
            httpretty.POST,
            f"https://api.opentok.com/v2/project/{self.api_key}/captions/7c0680fc-6274-4de5-a66f-d0648e8d3ac2/stop",
            status=202,
            content_type="application/json",
        )

        self.opentok.stop_captions(captions_id='7c0680fc-6274-4de5-a66f-d0648e8d3ac2')

        request_url = httpretty.last_request().url
        assert (
            request_url
            == 'https://api.opentok.com/v2/project/123456/captions/7c0680fc-6274-4de5-a66f-d0648e8d3ac2/stop'
        )
